@@ -16,7 +16,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  *
  */
 
@@ -24,12 +24,13 @@
  * VIA Padlock hardware acceleration.
  */
 
-#include <gnutls_int.h>
-#include <gnutls_hash_int.h>
-#include <gnutls_errors.h>
+#include "gnutls_int.h"
+#include <hash_int.h>
+#include "errors.h"
 #include <nettle/sha.h>
 #include <nettle/hmac.h>
 #include <nettle/macros.h>
+#include <nettle/memxor.h>
 #include <aes-padlock.h>
 #include <sha-padlock.h>
 #include <algorithms.h>
@@ -40,9 +41,9 @@
 #define OPAD 0x5c
 #define MAX_SHA_DIGEST_SIZE (512/8)
 
-typedef void (*update_func) (void *, unsigned, const uint8_t *);
-typedef void (*digest_func) (void *, unsigned, uint8_t *);
-typedef void (*set_key_func) (void *, unsigned, const uint8_t *);
+typedef void (*update_func) (void *, size_t, const uint8_t *);
+typedef void (*digest_func) (void *, size_t, uint8_t *);
+typedef void (*set_key_func) (void *, size_t, const uint8_t *);
 
 struct padlock_hmac_ctx {
 	union {
@@ -63,91 +64,91 @@ struct padlock_hmac_ctx {
 
 static void
 padlock_hmac_sha1_set_key(struct hmac_sha1_ctx *ctx,
-			  unsigned key_length, const uint8_t * key)
+			  size_t key_length, const uint8_t * key)
 {
 	HMAC_SET_KEY(ctx, &padlock_sha1, key_length, key);
 }
 
 static void
 padlock_hmac_sha1_update(struct hmac_sha1_ctx *ctx,
-			 unsigned length, const uint8_t * data)
+			 size_t length, const uint8_t * data)
 {
 	padlock_sha1_update(&ctx->state, length, data);
 }
 
 static void
 padlock_hmac_sha1_digest(struct hmac_sha1_ctx *ctx,
-			 unsigned length, uint8_t * digest)
+			 size_t length, uint8_t * digest)
 {
 	HMAC_DIGEST(ctx, &padlock_sha1, length, digest);
 }
 
 static void
 padlock_hmac_sha256_set_key(struct hmac_sha256_ctx *ctx,
-			    unsigned key_length, const uint8_t * key)
+			    size_t key_length, const uint8_t * key)
 {
 	HMAC_SET_KEY(ctx, &padlock_sha256, key_length, key);
 }
 
 static void
 padlock_hmac_sha256_update(struct hmac_sha256_ctx *ctx,
-			   unsigned length, const uint8_t * data)
+			   size_t length, const uint8_t * data)
 {
 	padlock_sha256_update(&ctx->state, length, data);
 }
 
 static void
 padlock_hmac_sha256_digest(struct hmac_sha256_ctx *ctx,
-			   unsigned length, uint8_t * digest)
+			   size_t length, uint8_t * digest)
 {
 	HMAC_DIGEST(ctx, &padlock_sha256, length, digest);
 }
 
 static void
 padlock_hmac_sha224_set_key(struct hmac_sha224_ctx *ctx,
-			    unsigned key_length, const uint8_t * key)
+			    size_t key_length, const uint8_t * key)
 {
 	HMAC_SET_KEY(ctx, &padlock_sha224, key_length, key);
 }
 
 static void
 padlock_hmac_sha224_digest(struct hmac_sha224_ctx *ctx,
-			   unsigned length, uint8_t * digest)
+			   size_t length, uint8_t * digest)
 {
 	HMAC_DIGEST(ctx, &padlock_sha224, length, digest);
 }
 
 static void
 padlock_hmac_sha384_set_key(struct hmac_sha384_ctx *ctx,
-			    unsigned key_length, const uint8_t * key)
+			    size_t key_length, const uint8_t * key)
 {
 	HMAC_SET_KEY(ctx, &padlock_sha384, key_length, key);
 }
 
 static void
 padlock_hmac_sha384_digest(struct hmac_sha384_ctx *ctx,
-			   unsigned length, uint8_t * digest)
+			   size_t length, uint8_t * digest)
 {
 	HMAC_DIGEST(ctx, &padlock_sha384, length, digest);
 }
 
 static void
 padlock_hmac_sha512_set_key(struct hmac_sha512_ctx *ctx,
-			    unsigned key_length, const uint8_t * key)
+			    size_t key_length, const uint8_t * key)
 {
 	HMAC_SET_KEY(ctx, &padlock_sha512, key_length, key);
 }
 
 static void
 padlock_hmac_sha512_update(struct hmac_sha512_ctx *ctx,
-			   unsigned length, const uint8_t * data)
+			   size_t length, const uint8_t * data)
 {
 	padlock_sha512_update(&ctx->state, length, data);
 }
 
 static void
 padlock_hmac_sha512_digest(struct hmac_sha512_ctx *ctx,
-			   unsigned length, uint8_t * digest)
+			   size_t length, uint8_t * digest)
 {
 	HMAC_DIGEST(ctx, &padlock_sha512, length, digest);
 }
