@@ -19,8 +19,6 @@
  * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
-#ifndef GNUTLS_LIB_ACCELERATED_X86_X86_COMMON_H
-#define GNUTLS_LIB_ACCELERATED_X86_X86_COMMON_H
 
 #include <config.h>
 
@@ -29,19 +27,24 @@
 void gnutls_cpuid(unsigned int func, unsigned int *ax, unsigned int *bx,
 		  unsigned int *cx, unsigned int *dx);
 
+# ifdef ASM_X86_32
+unsigned int gnutls_have_cpuid(void);
+# else
+#  define gnutls_have_cpuid() 1
+# endif				/* ASM_X86_32 */
+
 #endif
 
-#define CHECK_AES_KEYSIZE(s)               \
+#define CHECK_AES_KEYSIZE(s) \
 	if (s != 16 && s != 24 && s != 32) \
-	return GNUTLS_E_INVALID_REQUEST
+		return GNUTLS_E_INVALID_REQUEST
 
-#define NN_HASH(name, update_func, digest_func, NAME)                 \
-	{                                                             \
-		#name, sizeof(struct name##_ctx), NAME##_DIGEST_SIZE, \
-			NAME##_DATA_SIZE,                             \
-			(nettle_hash_init_func *)name##_init,         \
-			(nettle_hash_update_func *)update_func,       \
-			(nettle_hash_digest_func *)digest_func        \
-	}
-
-#endif
+#define NN_HASH(name, update_func, digest_func, NAME) {	\
+ #name,						\
+ sizeof(struct name##_ctx),			\
+ NAME##_DIGEST_SIZE,				\
+ NAME##_DATA_SIZE,				\
+ (nettle_hash_init_func *) name##_init,		\
+ (nettle_hash_update_func *) update_func,	\
+ (nettle_hash_digest_func *) digest_func	\
+} 
