@@ -16,7 +16,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  *
  */
 
@@ -54,7 +54,6 @@ void _gnutls_free_dh_info(dh_info_st * dh)
 int
 _gnutls_proc_dh_common_client_kx(gnutls_session_t session,
 				 uint8_t * data, size_t _data_size,
-				 bigint_t g, bigint_t p,
 				 gnutls_datum_t * psk_key)
 {
 	uint16_t n_Y;
@@ -75,17 +74,17 @@ _gnutls_proc_dh_common_client_kx(gnutls_session_t session,
 	if (data_size != 0)
 		return gnutls_assert_val(GNUTLS_E_UNEXPECTED_PACKET_LENGTH);
 
-	if (_gnutls_mpi_init_scan_nz(&session->key.client_Y, &data[2], _n_Y)) {
+	if (_gnutls_mpi_init_scan_nz(&session->key.proto.tls12.dh.client_Y, &data[2], _n_Y)) {
 		gnutls_assert();
-		return GNUTLS_E_MPI_SCAN_FAILED;
+		return GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER; /* most likely zero or illegal size */
 	}
 
-	_gnutls_dh_set_peer_public(session, session->key.client_Y);
+	_gnutls_dh_set_peer_public(session, session->key.proto.tls12.dh.client_Y);
 
-	peer_pub.params[DH_Y] = session->key.client_Y;
+	peer_pub.params[DH_Y] = session->key.proto.tls12.dh.client_Y;
 
 	/* calculate the key after calculating the message */
-	ret = _gnutls_pk_derive(GNUTLS_PK_DH, &tmp_dh_key, &session->key.dh_params, &peer_pub);
+	ret = _gnutls_pk_derive(GNUTLS_PK_DH, &tmp_dh_key, &session->key.proto.tls12.dh.params, &peer_pub);
 	if (ret < 0) {
 		gnutls_assert();
 		goto error;
@@ -107,9 +106,9 @@ _gnutls_proc_dh_common_client_kx(gnutls_session_t session,
 	}
 
 	ret = 0;
-error:
-	_gnutls_mpi_release(&session->key.client_Y);
-      	gnutls_pk_params_clear(&session->key.dh_params);
+ error:
+	_gnutls_mpi_release(&session->key.proto.tls12.dh.client_Y);
+	gnutls_pk_params_clear(&session->key.proto.tls12.dh.params);
 
 	return ret;
 }
@@ -128,35 +127,34 @@ _gnutls_gen_dh_common_client_kx_int(gnutls_session_t session,
 	int ret;
 	gnutls_pk_params_st peer_pub;
 	gnutls_datum_t tmp_dh_key = {NULL, 0};
+	unsigned init_pos = data->length;
 	
 	gnutls_pk_params_init(&peer_pub);
 
 	ret =
 	    _gnutls_pk_generate_keys(GNUTLS_PK_DH, 0,
-				     &session->key.dh_params);
+				     &session->key.proto.tls12.dh.params, 1);
 	if (ret < 0)
 		return gnutls_assert_val(ret);
 
-	_gnutls_dh_set_secret_bits(session, _gnutls_mpi_get_nbits(session->key.dh_params.params[DH_X]));
+	_gnutls_dh_set_secret_bits(session, _gnutls_mpi_get_nbits(session->key.proto.tls12.dh.params.params[DH_X]));
 
-	ret = _gnutls_buffer_append_mpi(data, 16, session->key.dh_params.params[DH_Y], 0);
+	ret = _gnutls_buffer_append_mpi(data, 16, session->key.proto.tls12.dh.params.params[DH_Y], 0);
 	if (ret < 0) {
 		gnutls_assert();
 		goto error;
 	}
 	
-	peer_pub.params[DH_Y] = session->key.client_Y;
+	peer_pub.params[DH_Y] = session->key.proto.tls12.dh.client_Y;
 
 	/* calculate the key after calculating the message */
-	ret = _gnutls_pk_derive(GNUTLS_PK_DH, &tmp_dh_key, &session->key.dh_params, &peer_pub);
+	ret = _gnutls_pk_derive(GNUTLS_PK_DH, &tmp_dh_key, &session->key.proto.tls12.dh.params, &peer_pub);
 	if (ret < 0) {
 		gnutls_assert();
 		goto error;
 	}
 
-	if (_gnutls_cipher_suite_get_kx_algo
-	    (session->security_parameters.cipher_suite)
-	    != GNUTLS_KX_DHE_PSK) {
+	if (session->security_parameters.cs->kx_algorithm != GNUTLS_KX_DHE_PSK) {
 		session->key.key.data = tmp_dh_key.data;
 		session->key.key.size = tmp_dh_key.size;
 	} else {		/* In DHE_PSK the key is set differently */
@@ -171,38 +169,13 @@ _gnutls_gen_dh_common_client_kx_int(gnutls_session_t session,
 		goto error;
 	}
 
-	ret = data->length;
+	ret = data->length - init_pos;
 
-      error:
-      	gnutls_pk_params_clear(&session->key.dh_params);
+ error:
+	gnutls_pk_params_clear(&session->key.proto.tls12.dh.params);
 	return ret;
 }
 
-int _gnutls_set_dh_pk_params(gnutls_session_t session, bigint_t g, bigint_t p,
-				unsigned q_bits)
-{
-	/* just in case we are resuming a session */
-	gnutls_pk_params_release(&session->key.dh_params);
-
-	gnutls_pk_params_init(&session->key.dh_params);
-
-	session->key.dh_params.params[DH_G] = _gnutls_mpi_copy(g);
-	if (session->key.dh_params.params[DH_G] == NULL)
-		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
-	
-	session->key.dh_params.params[DH_P] = _gnutls_mpi_copy(p);
-	if (session->key.dh_params.params[DH_P] == NULL) {
-		_gnutls_mpi_release(&session->key.dh_params.params[DH_G]);
-		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
-	}
-
-	session->key.dh_params.params_nr = 3; /* include empty q */
-	session->key.dh_params.algo = GNUTLS_PK_DH;
-	session->key.dh_params.flags = q_bits;
-	
-	return 0;
-}
-
 /* Returns the bytes parsed */
 int
 _gnutls_proc_dh_common_server_kx(gnutls_session_t session,
@@ -214,12 +187,13 @@ _gnutls_proc_dh_common_server_kx(gnutls_session_t session,
 	uint8_t *data_g;
 	uint8_t *data_Y;
 	int i, bits, ret, p_bits;
+	unsigned j;
 	ssize_t data_size = _data_size;
-	
-	/* just in case we are resuming a session */
-	gnutls_pk_params_release(&session->key.dh_params);
 
-	gnutls_pk_params_init(&session->key.dh_params);
+	/* just in case we are resuming a session */
+	gnutls_pk_params_release(&session->key.proto.tls12.dh.params);
+
+	gnutls_pk_params_init(&session->key.proto.tls12.dh.params);
 
 	i = 0;
 
@@ -250,53 +224,83 @@ _gnutls_proc_dh_common_server_kx(gnutls_session_t session,
 	_n_g = n_g;
 	_n_p = n_p;
 
-	if (_gnutls_mpi_init_scan_nz(&session->key.client_Y, data_Y, _n_Y) != 0) {
+	if (_gnutls_mpi_init_scan_nz(&session->key.proto.tls12.dh.client_Y, data_Y, _n_Y) != 0) {
 		gnutls_assert();
-		return GNUTLS_E_MPI_SCAN_FAILED;
+		return GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER;
 	}
 
-	if (_gnutls_mpi_init_scan_nz(&session->key.dh_params.params[DH_G], data_g, _n_g) != 0) {
-		gnutls_assert();
-		return GNUTLS_E_MPI_SCAN_FAILED;
+	/* if we are doing RFC7919 */
+	if (session->internals.priorities->groups.have_ffdhe != 0) {
+		/* verify whether the received parameters match the advertised, otherwise
+		 * log that. */
+		for (j=0;j<session->internals.priorities->groups.size;j++) {
+			if (session->internals.priorities->groups.entry[j]->generator &&
+			    session->internals.priorities->groups.entry[j]->generator->size == n_g &&
+			    session->internals.priorities->groups.entry[j]->prime->size == n_p &&
+			    memcmp(session->internals.priorities->groups.entry[j]->generator->data,
+				   data_g, n_g) == 0 &&
+			    memcmp(session->internals.priorities->groups.entry[j]->prime->data,
+				   data_p, n_p) == 0) {
+
+				session->internals.hsk_flags |= HSK_USED_FFDHE;
+				_gnutls_session_group_set(session, session->internals.priorities->groups.entry[j]);
+				session->key.proto.tls12.dh.params.qbits = *session->internals.priorities->groups.entry[j]->q_bits;
+				break;
+			}
+		}
+
+		if (!(session->internals.hsk_flags & HSK_USED_FFDHE)) {
+			_gnutls_audit_log(session, "FFDHE groups advertised, but server didn't support it; falling back to server's choice\n");
+		}
 	}
 
-	if (_gnutls_mpi_init_scan_nz(&session->key.dh_params.params[DH_P], data_p, _n_p) != 0) {
+	if (_gnutls_mpi_init_scan_nz(&session->key.proto.tls12.dh.params.params[DH_G], data_g, _n_g) != 0) {
 		gnutls_assert();
-		return GNUTLS_E_MPI_SCAN_FAILED;
-	}
-	session->key.dh_params.params_nr = 3; /* include empty q */
-	session->key.dh_params.algo = GNUTLS_PK_DH;
-
-	bits = _gnutls_dh_get_min_prime_bits(session);
-	if (bits < 0) {
-		gnutls_assert();
-		return bits;
+		return GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER;
 	}
 
-	p_bits = _gnutls_mpi_get_nbits(session->key.dh_params.params[DH_P]);
-	if (p_bits < bits) {
-		/* the prime used by the peer is not acceptable
-		 */
+	if (_gnutls_mpi_init_scan_nz(&session->key.proto.tls12.dh.params.params[DH_P], data_p, _n_p) != 0) {
 		gnutls_assert();
-		_gnutls_debug_log
-		    ("Received a prime of %u bits, limit is %u\n",
-		     (unsigned) _gnutls_mpi_get_nbits(session->key.dh_params.params[DH_P]),
-		     (unsigned) bits);
-		return GNUTLS_E_DH_PRIME_UNACCEPTABLE;
+		/* we release now because session->key.proto.tls12.dh.params.params_nr is not yet set */
+		_gnutls_mpi_release(&session->key.proto.tls12.dh.params.params[DH_G]);
+		return GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER;
 	}
 
-	if (p_bits >= DEFAULT_MAX_VERIFY_BITS) {
-		gnutls_assert();
-		_gnutls_debug_log
-		    ("Received a prime of %u bits, limit is %u\n",
-		     (unsigned) p_bits,
-		     (unsigned) DEFAULT_MAX_VERIFY_BITS);
-		return GNUTLS_E_DH_PRIME_UNACCEPTABLE;
+	session->key.proto.tls12.dh.params.params_nr = 3; /* include empty q */
+	session->key.proto.tls12.dh.params.algo = GNUTLS_PK_DH;
+
+	if (!(session->internals.hsk_flags & HSK_USED_FFDHE)) {
+		bits = _gnutls_dh_get_min_prime_bits(session);
+		if (bits < 0) {
+			gnutls_assert();
+			return bits;
+		}
+
+		p_bits = _gnutls_mpi_get_nbits(session->key.proto.tls12.dh.params.params[DH_P]);
+		if (p_bits < bits) {
+			/* the prime used by the peer is not acceptable
+			 */
+			gnutls_assert();
+			_gnutls_debug_log
+			    ("Received a prime of %u bits, limit is %u\n",
+			     (unsigned) _gnutls_mpi_get_nbits(session->key.proto.tls12.dh.params.params[DH_P]),
+			     (unsigned) bits);
+			return GNUTLS_E_DH_PRIME_UNACCEPTABLE;
+		}
+
+		if (p_bits >= DEFAULT_MAX_VERIFY_BITS) {
+			gnutls_assert();
+			_gnutls_debug_log
+			    ("Received a prime of %u bits, limit is %u\n",
+			     (unsigned) p_bits,
+			     (unsigned) DEFAULT_MAX_VERIFY_BITS);
+			return GNUTLS_E_DH_PRIME_UNACCEPTABLE;
+		}
 	}
 
-	_gnutls_dh_set_group(session, session->key.dh_params.params[DH_G],
-			     session->key.dh_params.params[DH_P]);
-	_gnutls_dh_set_peer_public(session, session->key.client_Y);
+	_gnutls_dh_save_group(session, session->key.proto.tls12.dh.params.params[DH_G],
+			     session->key.proto.tls12.dh.params.params[DH_P]);
+	_gnutls_dh_set_peer_public(session, session->key.proto.tls12.dh.client_Y);
 
 	ret = n_Y + n_p + n_g + 6;
 
@@ -308,7 +312,8 @@ _gnutls_dh_common_print_server_kx(gnutls_session_t session,
 				  gnutls_buffer_st * data)
 {
 	int ret;
-	unsigned q_bits = session->key.dh_params.flags;
+	unsigned q_bits = session->key.proto.tls12.dh.params.qbits;
+	unsigned init_pos = data->length;
 
 	if (q_bits < 192 && q_bits != 0) {
 		gnutls_assert();
@@ -319,31 +324,31 @@ _gnutls_dh_common_print_server_kx(gnutls_session_t session,
 	/* Y=g^x mod p */
 	ret =
 	    _gnutls_pk_generate_keys(GNUTLS_PK_DH, q_bits,
-				     &session->key.dh_params);
+				     &session->key.proto.tls12.dh.params, 1);
 	if (ret < 0)
 		return gnutls_assert_val(ret);
 
-	_gnutls_dh_set_secret_bits(session, _gnutls_mpi_get_nbits(session->key.dh_params.params[DH_X]));
+	_gnutls_dh_set_secret_bits(session, _gnutls_mpi_get_nbits(session->key.proto.tls12.dh.params.params[DH_X]));
 
-	ret = _gnutls_buffer_append_mpi(data, 16, session->key.dh_params.params[DH_P], 0);
+	ret = _gnutls_buffer_append_mpi(data, 16, session->key.proto.tls12.dh.params.params[DH_P], 0);
 	if (ret < 0) {
 		gnutls_assert();
 		goto cleanup;
 	}
 
-	ret = _gnutls_buffer_append_mpi(data, 16, session->key.dh_params.params[DH_G], 0);
+	ret = _gnutls_buffer_append_mpi(data, 16, session->key.proto.tls12.dh.params.params[DH_G], 0);
 	if (ret < 0) {
 		gnutls_assert();
 		goto cleanup;
 	}
 
-	ret = _gnutls_buffer_append_mpi(data, 16, session->key.dh_params.params[DH_Y], 0);
+	ret = _gnutls_buffer_append_mpi(data, 16, session->key.proto.tls12.dh.params.params[DH_Y], 0);
 	if (ret < 0) {
 		gnutls_assert();
 		goto cleanup;
 	}
 
-	ret = data->length;
+	ret = data->length - init_pos;
 
 cleanup:
 	return ret;
