@@ -27,42 +27,43 @@
 
 #include "gnutls_int.h"
 #include "errors.h"
-#include "auth/cert.h"
-#include "datum.h"
-#include "mpi.h"
-#include "global.h"
-#include "algorithms.h"
-#include "dh.h"
+#include <auth/cert.h>
+#include <datum.h>
+#include <mpi.h>
+#include <global.h>
+#include <algorithms.h>
+#include <dh.h>
 #include "str.h"
-#include "state.h"
-#include "auth.h"
-#include "x509.h"
-#include "str_array.h"
-#include "x509/verify-high.h"
+#include <state.h>
+#include <auth.h>
+#include <x509.h>
+#include <str_array.h>
+#include <x509/verify-high.h>
 #include "x509/x509_int.h"
 #include "x509/common.h"
 #include "dh.h"
 #include "cert-cred.h"
-#include "intprops.h"
+
 
 /*
  * Adds a public/private key pair to a certificate credential
  */
-int _gnutls_certificate_credential_append_keypair(
-	gnutls_certificate_credentials_t res, gnutls_privkey_t key,
-	gnutls_str_array_t names, gnutls_pcert_st *crt, int nr)
+int
+_gnutls_certificate_credential_append_keypair(gnutls_certificate_credentials_t res,
+				       gnutls_privkey_t key,
+				       gnutls_str_array_t names,
+				       gnutls_pcert_st * crt,
+				       int nr)
 {
-	if (unlikely(INT_ADD_OVERFLOW(res->ncerts, 1))) {
-		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
-	}
-
-	res->sorted_cert_idx = _gnutls_reallocarray_fast(
-		res->sorted_cert_idx, res->ncerts + 1, sizeof(unsigned int));
+	res->sorted_cert_idx = gnutls_realloc_fast(res->sorted_cert_idx,
+						(1 + res->ncerts) *
+						sizeof(unsigned int));
 	if (res->sorted_cert_idx == NULL)
 		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 
-	res->certs = _gnutls_reallocarray_fast(res->certs, res->ncerts + 1,
-					       sizeof(certs_st));
+	res->certs = gnutls_realloc_fast(res->certs,
+					 (1 + res->ncerts) *
+					 sizeof(certs_st));
 	if (res->certs == NULL)
 		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 
@@ -73,21 +74,17 @@ int _gnutls_certificate_credential_append_keypair(
 	res->certs[res->ncerts].names = names;
 	res->certs[res->ncerts].pkey = key;
 
-	if (_gnutls13_sign_get_compatible_with_privkey(key))
-		res->tls13_ok = 1;
-
 	/* move RSA-PSS certificates before any RSA key.
 	 * Note that we cannot assume that any previous pointers
 	 * to sorted list are ok, due to the realloc in res->certs. */
 	if (crt->pubkey->params.algo == GNUTLS_PK_RSA_PSS) {
-		unsigned i, ridx;
+		unsigned i,ridx;
 		unsigned tmp;
 
-		for (i = 0; i < res->ncerts; i++) {
+		for (i=0;i<res->ncerts;i++) {
 			ridx = res->sorted_cert_idx[i];
 
-			if (res->certs[ridx].cert_list->pubkey->params.algo ==
-			    GNUTLS_PK_RSA) {
+			if (res->certs[ridx].cert_list->pubkey->params.algo == GNUTLS_PK_RSA) {
 				tmp = ridx;
 				res->sorted_cert_idx[i] = res->ncerts;
 				res->sorted_cert_idx[res->ncerts] = tmp;
@@ -99,10 +96,12 @@ int _gnutls_certificate_credential_append_keypair(
 	/* otherwise append it normally on the end */
 	res->sorted_cert_idx[res->ncerts] = res->ncerts;
 
-finish:
+ finish:
 	return 0;
+
 }
 
+
 /**
  * gnutls_certificate_set_key:
  * @res: is a #gnutls_certificate_credentials_t type.
@@ -137,10 +136,13 @@ finish:
  *
  * Since: 3.0
  **/
-int gnutls_certificate_set_key(gnutls_certificate_credentials_t res,
-			       const char **names, int names_size,
-			       gnutls_pcert_st *pcert_list, int pcert_list_size,
-			       gnutls_privkey_t key)
+int
+gnutls_certificate_set_key(gnutls_certificate_credentials_t res,
+			   const char **names,
+			   int names_size,
+			   gnutls_pcert_st * pcert_list,
+			   int pcert_list_size,
+			   gnutls_privkey_t key)
 {
 	int ret, i;
 	gnutls_str_array_t str_names;
@@ -151,6 +153,7 @@ int gnutls_certificate_set_key(gnutls_certificate_credentials_t res,
 	if (res == NULL) {
 		return gnutls_assert_val(GNUTLS_E_ILLEGAL_PARAMETER);
 	}
+
 	// A complete key pair must be given
 	if (pcert_list == NULL || key == NULL) {
 		return gnutls_assert_val(GNUTLS_E_INSUFFICIENT_CREDENTIALS);
@@ -161,8 +164,9 @@ int gnutls_certificate_set_key(gnutls_certificate_credentials_t res,
 
 	if (names != NULL && names_size > 0) {
 		for (i = 0; i < names_size; i++) {
-			ret = _gnutls_str_array_append_idna(
-				&str_names, names[i], strlen(names[i]));
+			ret =
+			    _gnutls_str_array_append_idna(&str_names, names[i],
+						     strlen(names[i]));
 			if (ret < 0) {
 				ret = gnutls_assert_val(ret);
 				goto cleanup;
@@ -177,8 +181,7 @@ int gnutls_certificate_set_key(gnutls_certificate_credentials_t res,
 			goto cleanup;
 		}
 
-		ret = gnutls_x509_crt_import(crt, &pcert_list[0].cert,
-					     GNUTLS_X509_FMT_DER);
+		ret = gnutls_x509_crt_import(crt, &pcert_list[0].cert, GNUTLS_X509_FMT_DER);
 		if (ret < 0) {
 			gnutls_assert();
 			gnutls_x509_crt_deinit(crt);
@@ -198,16 +201,16 @@ int gnutls_certificate_set_key(gnutls_certificate_credentials_t res,
 		gnutls_privkey_set_pin_function(key, res->pin.cb,
 						res->pin.data);
 
-	new_pcert_list = _gnutls_reallocarray(NULL, pcert_list_size,
-					      sizeof(gnutls_pcert_st));
+	new_pcert_list = gnutls_malloc(sizeof(gnutls_pcert_st) * pcert_list_size);
 	if (new_pcert_list == NULL) {
 		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 	}
-	memcpy(new_pcert_list, pcert_list,
-	       sizeof(gnutls_pcert_st) * pcert_list_size);
+	memcpy(new_pcert_list, pcert_list, sizeof(gnutls_pcert_st) * pcert_list_size);
 
-	ret = _gnutls_certificate_credential_append_keypair(
-		res, key, str_names, new_pcert_list, pcert_list_size);
+	ret =
+	    _gnutls_certificate_credential_append_keypair(res, key, str_names,
+						   new_pcert_list,
+						   pcert_list_size);
 	if (ret < 0) {
 		gnutls_assert();
 		gnutls_free(new_pcert_list);
@@ -229,7 +232,7 @@ int gnutls_certificate_set_key(gnutls_certificate_credentials_t res,
 
 	CRED_RET_SUCCESS(res);
 
-cleanup:
+      cleanup:
 	_gnutls_str_array_clear(&str_names);
 	return ret;
 }
@@ -301,9 +304,11 @@ void gnutls_certificate_free_cas(gnutls_certificate_credentials_t sc)
  *
  * Since: 3.0
  **/
-int gnutls_certificate_get_issuer(gnutls_certificate_credentials_t sc,
-				  gnutls_x509_crt_t cert,
-				  gnutls_x509_crt_t *issuer, unsigned int flags)
+int
+gnutls_certificate_get_issuer(gnutls_certificate_credentials_t sc,
+			      gnutls_x509_crt_t cert,
+			      gnutls_x509_crt_t * issuer,
+			      unsigned int flags)
 {
 	return gnutls_x509_trust_list_get_issuer(sc->tlist, cert, issuer,
 						 flags);
@@ -328,15 +333,20 @@ int gnutls_certificate_get_issuer(gnutls_certificate_credentials_t sc,
  *
  * Since: 3.2.5
  **/
-int gnutls_certificate_get_crt_raw(gnutls_certificate_credentials_t sc,
-				   unsigned idx1, unsigned idx2,
-				   gnutls_datum_t *cert)
+int
+gnutls_certificate_get_crt_raw(gnutls_certificate_credentials_t sc,
+			       unsigned idx1,
+			       unsigned idx2, gnutls_datum_t * cert)
 {
 	if (idx1 >= sc->ncerts)
-		return gnutls_assert_val(GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE);
+		return
+		    gnutls_assert_val
+		    (GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE);
 
 	if (idx2 >= sc->certs[idx1].cert_list_length)
-		return gnutls_assert_val(GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE);
+		return
+		    gnutls_assert_val
+		    (GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE);
 
 	cert->data = sc->certs[idx1].cert_list[idx2].cert.data;
 	cert->size = sc->certs[idx1].cert_list[idx2].cert.size;
@@ -363,6 +373,7 @@ void gnutls_certificate_free_ca_names(gnutls_certificate_credentials_t sc)
 	_gnutls_free_datum(&sc->tlist->x509_rdn_sequence);
 }
 
+
 /**
  * gnutls_certificate_free_credentials:
  * @sc: is a #gnutls_certificate_credentials_t type.
@@ -373,7 +384,8 @@ void gnutls_certificate_free_ca_names(gnutls_certificate_credentials_t sc)
  * with this structure (ie RSA and DH parameters are not freed by this
  * function).
  **/
-void gnutls_certificate_free_credentials(gnutls_certificate_credentials_t sc)
+void
+gnutls_certificate_free_credentials(gnutls_certificate_credentials_t sc)
 {
 	// Check for valid pointer and otherwise do nothing
 	if (sc == NULL)
@@ -390,6 +402,7 @@ void gnutls_certificate_free_credentials(gnutls_certificate_credentials_t sc)
 	gnutls_free(sc);
 }
 
+
 /**
  * gnutls_certificate_allocate_credentials:
  * @res: is a pointer to a #gnutls_certificate_credentials_t type.
@@ -398,8 +411,9 @@ void gnutls_certificate_free_credentials(gnutls_certificate_credentials_t sc)
  *
  * Returns: %GNUTLS_E_SUCCESS on success, or an error code.
  **/
-int gnutls_certificate_allocate_credentials(
-	gnutls_certificate_credentials_t *res)
+int
+gnutls_certificate_allocate_credentials(gnutls_certificate_credentials_t *
+					res)
 {
 	int ret;
 
@@ -417,30 +431,24 @@ int gnutls_certificate_allocate_credentials(
 	(*res)->verify_bits = DEFAULT_MAX_VERIFY_BITS;
 	(*res)->verify_depth = DEFAULT_MAX_VERIFY_DEPTH;
 
+
 	return 0;
 }
 
 /* converts the given x509 certificate list to gnutls_pcert_st* and allocates
  * space for them.
  */
-static gnutls_pcert_st *alloc_and_load_x509_certs(gnutls_x509_crt_t *certs,
-						  unsigned ncerts)
+static gnutls_pcert_st *alloc_and_load_x509_certs(gnutls_x509_crt_t *
+						  certs, unsigned ncerts)
 {
 	gnutls_pcert_st *local_certs;
 	int ret = 0;
 	unsigned i, j;
 
-	if (ncerts == 0) {
+	if (certs == NULL)
 		return NULL;
-	}
 
-	if (unlikely(certs == NULL)) {
-		gnutls_assert();
-		return NULL;
-	}
-
-	local_certs =
-		_gnutls_reallocarray(NULL, ncerts, sizeof(gnutls_pcert_st));
+	local_certs = gnutls_malloc(sizeof(gnutls_pcert_st) * ncerts);
 	if (local_certs == NULL) {
 		gnutls_assert();
 		return NULL;
@@ -467,8 +475,8 @@ static gnutls_pcert_st *alloc_and_load_x509_certs(gnutls_x509_crt_t *certs,
 /* converts the given x509 key to gnutls_privkey* and allocates
  * space for it.
  */
-static gnutls_privkey_t alloc_and_load_x509_key(gnutls_x509_privkey_t key,
-						int deinit)
+static gnutls_privkey_t
+alloc_and_load_x509_key(gnutls_x509_privkey_t key, int deinit)
 {
 	gnutls_privkey_t local_key;
 	int ret = 0;
@@ -482,9 +490,10 @@ static gnutls_privkey_t alloc_and_load_x509_key(gnutls_x509_privkey_t key,
 		return NULL;
 	}
 
-	ret = gnutls_privkey_import_x509(
-		local_key, key,
-		deinit ? GNUTLS_PRIVKEY_IMPORT_AUTO_RELEASE : 0);
+	ret =
+	    gnutls_privkey_import_x509(local_key, key,
+				       deinit ?
+				       GNUTLS_PRIVKEY_IMPORT_AUTO_RELEASE : 0);
 	if (ret < 0) {
 		gnutls_assert();
 		gnutls_privkey_deinit(local_key);
@@ -499,8 +508,8 @@ static gnutls_privkey_t alloc_and_load_x509_key(gnutls_x509_privkey_t key,
 /* converts the given raw key to gnutls_privkey* and allocates
  * space for it.
  */
-static gnutls_privkey_t alloc_and_load_pkcs11_key(gnutls_pkcs11_privkey_t key,
-						  int deinit)
+static gnutls_privkey_t
+alloc_and_load_pkcs11_key(gnutls_pkcs11_privkey_t key, int deinit)
 {
 	gnutls_privkey_t local_key;
 	int ret = 0;
@@ -514,9 +523,11 @@ static gnutls_privkey_t alloc_and_load_pkcs11_key(gnutls_pkcs11_privkey_t key,
 		return NULL;
 	}
 
-	ret = gnutls_privkey_import_pkcs11(
-		local_key, key,
-		deinit ? GNUTLS_PRIVKEY_IMPORT_AUTO_RELEASE : 0);
+	ret =
+	    gnutls_privkey_import_pkcs11(local_key, key,
+					 deinit ?
+					 GNUTLS_PRIVKEY_IMPORT_AUTO_RELEASE
+					 : 0);
 	if (ret < 0) {
 		gnutls_assert();
 		gnutls_privkey_deinit(local_key);
@@ -531,17 +542,17 @@ static gnutls_privkey_t alloc_and_load_pkcs11_key(gnutls_pkcs11_privkey_t key,
 /**
  * gnutls_certificate_server_set_request:
  * @session: is a #gnutls_session_t type.
- * @req: is one of GNUTLS_CERT_REQUEST, GNUTLS_CERT_REQUIRE, GNUTLS_CERT_IGNORE
+ * @req: is one of GNUTLS_CERT_REQUEST, GNUTLS_CERT_REQUIRE
  *
  * This function specifies if we (in case of a server) are going to
  * send a certificate request message to the client. If @req is
  * GNUTLS_CERT_REQUIRE then the server will return the %GNUTLS_E_NO_CERTIFICATE_FOUND
  * error if the peer does not provide a certificate. If you do not call this
- * function then the client will not be asked to send a certificate. Invoking
- * the function with @req GNUTLS_CERT_IGNORE has the same effect.
+ * function then the client will not be asked to send a certificate.
  **/
-void gnutls_certificate_server_set_request(gnutls_session_t session,
-					   gnutls_certificate_request_t req)
+void
+gnutls_certificate_server_set_request(gnutls_session_t session,
+				      gnutls_certificate_request_t req)
 {
 	session->internals.send_cert_req = req;
 }
@@ -552,7 +563,8 @@ static int call_legacy_cert_cb1(gnutls_session_t session,
 				unsigned int *pcert_length,
 				gnutls_ocsp_data_st **ocsp,
 				unsigned int *ocsp_length,
-				gnutls_privkey_t *privkey, unsigned int *flags)
+				gnutls_privkey_t *privkey,
+				unsigned int *flags)
 {
 	gnutls_retr2_st st2;
 	gnutls_pcert_st *local_certs = NULL;
@@ -564,9 +576,9 @@ static int call_legacy_cert_cb1(gnutls_session_t session,
 
 	memset(&st2, 0, sizeof(st2));
 
-	ret = info->cred->legacy_cert_cb1(session, info->req_ca_rdn,
-					  info->nreqs, info->pk_algos,
-					  info->pk_algos_length, &st2);
+	ret = info->cred->legacy_cert_cb1(session, info->req_ca_rdn, info->nreqs,
+					  info->pk_algos, info->pk_algos_length,
+				          &st2);
 	if (ret < 0)
 		return gnutls_assert_val(ret);
 
@@ -583,7 +595,8 @@ static int call_legacy_cert_cb1(gnutls_session_t session,
 		goto cleanup;
 	}
 
-	local_certs = alloc_and_load_x509_certs(st2.cert.x509, st2.ncerts);
+	local_certs =
+	    alloc_and_load_x509_certs(st2.cert.x509, st2.ncerts);
 	if (local_certs == NULL) {
 		gnutls_assert();
 		ret = GNUTLS_E_MEMORY_ERROR;
@@ -594,8 +607,9 @@ static int call_legacy_cert_cb1(gnutls_session_t session,
 #ifdef ENABLE_PKCS11
 	case GNUTLS_PRIVKEY_PKCS11:
 		if (st2.key.pkcs11 != NULL) {
-			local_key = alloc_and_load_pkcs11_key(st2.key.pkcs11,
-							      st2.deinit_all);
+			local_key =
+			    alloc_and_load_pkcs11_key(st2.key.pkcs11,
+						      st2.deinit_all);
 			if (local_key == NULL) {
 				gnutls_assert();
 				ret = GNUTLS_E_INTERNAL_ERROR;
@@ -606,8 +620,9 @@ static int call_legacy_cert_cb1(gnutls_session_t session,
 #endif
 	case GNUTLS_PRIVKEY_X509:
 		if (st2.key.x509 != NULL) {
-			local_key = alloc_and_load_x509_key(st2.key.x509,
-							    st2.deinit_all);
+			local_key =
+			    alloc_and_load_x509_key(st2.key.x509,
+						    st2.deinit_all);
 			if (local_key == NULL) {
 				gnutls_assert();
 				ret = GNUTLS_E_INTERNAL_ERROR;
@@ -630,7 +645,7 @@ static int call_legacy_cert_cb1(gnutls_session_t session,
 
 	ret = 0;
 
-cleanup:
+ cleanup:
 
 	if (st2.cert_type == GNUTLS_CRT_X509) {
 		if (st2.deinit_all) {
@@ -642,6 +657,7 @@ cleanup:
 	}
 
 	return ret;
+
 }
 
 /**
@@ -684,9 +700,9 @@ cleanup:
  *
  * Since: 3.0
  **/
-void gnutls_certificate_set_retrieve_function(
-	gnutls_certificate_credentials_t cred,
-	gnutls_certificate_retrieve_function *func)
+void gnutls_certificate_set_retrieve_function
+    (gnutls_certificate_credentials_t cred,
+     gnutls_certificate_retrieve_function * func)
 {
 	cred->legacy_cert_cb1 = func;
 	if (!func)
@@ -701,16 +717,16 @@ static int call_legacy_cert_cb2(gnutls_session_t session,
 				unsigned int *pcert_length,
 				gnutls_ocsp_data_st **ocsp,
 				unsigned int *ocsp_length,
-				gnutls_privkey_t *privkey, unsigned int *flags)
+				gnutls_privkey_t *privkey,
+				unsigned int *flags)
 {
 	int ret;
 	*ocsp_length = 0;
 	/* flags will be assumed to be zero */
 
-	ret = info->cred->legacy_cert_cb2(session, info->req_ca_rdn,
-					  info->nreqs, info->pk_algos,
-					  info->pk_algos_length, certs,
-					  pcert_length, privkey);
+	ret = info->cred->legacy_cert_cb2(session, info->req_ca_rdn, info->nreqs,
+					  info->pk_algos, info->pk_algos_length,
+					  certs, pcert_length, privkey);
 	if (ret < 0) {
 		gnutls_assert();
 	}
@@ -762,9 +778,9 @@ static int call_legacy_cert_cb2(gnutls_session_t session,
  *
  * Since: 3.0
  **/
-void gnutls_certificate_set_retrieve_function2(
-	gnutls_certificate_credentials_t cred,
-	gnutls_certificate_retrieve_function2 *func)
+void gnutls_certificate_set_retrieve_function2
+    (gnutls_certificate_credentials_t cred,
+     gnutls_certificate_retrieve_function2 * func)
 {
 	cred->legacy_cert_cb2 = func;
 	if (!func)
@@ -784,16 +800,10 @@ void gnutls_certificate_set_retrieve_function2(
  * or during post-handshake authentication.
  *
  * The callback's function prototype is defined in `abstract.h':
- *
- * int gnutls_certificate_retrieve_function3(
- * gnutls_session_t,
- * const struct gnutls_cert_retr_st *info,
- * gnutls_pcert_st **certs,
- * unsigned int *certs_length,
- * gnutls_ocsp_data_st **ocsp,
- * unsigned int *ocsp_length,
- * gnutls_privkey_t *privkey,
- * unsigned int *flags);
+ * int (*callback)(gnutls_session_t, const struct gnutls_cert_retr_st *info,
+ * gnutls_pcert_st **certs, unsigned int *pcert_length,
+ * gnutls_datum_t **ocsp, unsigned int *ocsp_length,
+ * gnutls_privkey_t * pkey, unsigned int *flags);
  *
  * The info field of the callback contains:
  * @req_ca_dn which is a list with the CA names that the server considers trusted.
@@ -803,13 +813,13 @@ void gnutls_certificate_set_retrieve_function2(
  * @pk_algos contains a list with server's acceptable public key algorithms.
  * The certificate returned should support the server's given algorithms.
  *
- * The callback should fill-in the following values:
+ * The callback should fill-in the following values.
  *
- * @certs should contain an allocated list of certificates and public keys.
- * @certs_length is the size of the previous list.
+ * @pcert should contain an allocated list of certificates and public keys.
+ * @pcert_length is the size of the previous list.
  * @ocsp should contain an allocated list of OCSP responses.
  * @ocsp_length is the size of the previous list.
- * @privkey is the private key.
+ * @pkey is the private key.
  *
  * If flags in the callback are set to %GNUTLS_CERT_RETR_DEINIT_ALL then
  * all provided values must be allocated using gnutls_malloc(), and will
@@ -817,23 +827,16 @@ void gnutls_certificate_set_retrieve_function2(
  *
  * The callback function should set the certificate and OCSP response
  * list to be sent, and return 0 on success. If no certificates are available,
- * the @certs_length and @ocsp_length should be set to zero. The return
+ * the @pcert_length and @ocsp_length should be set to zero. The return
  * value (-1) indicates error and the handshake will be terminated. If both
  * certificates are set in the credentials and a callback is available, the
  * callback takes predence.
  *
- * Raw public-keys:
- * In case raw public-keys are negotiated as certificate type, certificates
- * that would normally hold the public-key material are not available. In that case,
- * @certs contains an allocated list with only the public key. Since there is no
- * certificate, there is also no certificate status. Therefore, OCSP information
- * should not be set.
- *
  * Since: 3.6.3
  **/
-void gnutls_certificate_set_retrieve_function3(
-	gnutls_certificate_credentials_t cred,
-	gnutls_certificate_retrieve_function3 *func)
+void gnutls_certificate_set_retrieve_function3
+    (gnutls_certificate_credentials_t cred,
+     gnutls_certificate_retrieve_function3 *func)
 {
 	cred->get_cert_callback3 = func;
 }
@@ -861,130 +864,22 @@ void gnutls_certificate_set_retrieve_function3(
  *
  * Since: 2.10.0
  **/
-void gnutls_certificate_set_verify_function(
-	gnutls_certificate_credentials_t cred,
-	gnutls_certificate_verify_function *func)
+void
+ gnutls_certificate_set_verify_function
+    (gnutls_certificate_credentials_t cred,
+     gnutls_certificate_verify_function * func)
 {
 	cred->verify_callback = func;
 }
 
-/**
- * gnutls_x509_trust_list_set_getissuer_function:
- * @tlist: is a #gnutls_x509_trust_list_t type.
- * @func: is the callback function
- *
- * This function sets a callback to be called when the peer's certificate
- * chain is incomplete due a missing intermediate certificate. The callback
- * may provide the missing certificate for use during verification.
- *
- * The callback's function prototype is defined in gnutls/x509.h as:
- *
- *   int (*callback)(gnutls_x509_trust_list_t list,
- *                   const gnutls_x509_crt_t cert,
- *                   gnutls_x509_crt_t **issuers,
- *                   unsigned int *issuers_size);
- *
- * If the callback function is provided then gnutls will call it during the
- * certificate verification procedure. The callback may wish to use
- * gnutls_x509_crt_get_authority_info_access() to get a URI from which
- * to attempt to download the missing issuer certificate, if available.
- *
- * On a successful call, the callback shall set '*issuers' and '*issuers_size'
- * even if the result is empty; in that case '*issuers' will point to %NULL and
- * '*issuers_size' will be 0.  Otherwise, the '*issuers' array shall be
- * allocated using gnutls_x509_crt_list_import2(). The ownership of both the
- * array and the elements is transferred to the caller and thus the application
- * does not need to maintain the memory after the call.
- *
- * The callback function should return 0 if the attempt to retrieve the issuer
- * certificates for 'crt' succeeded, or non-zero to indicate any error occurred
- * during the attempt. In the latter case, '*issuers' and '*issuers_size' are
- * not set.
- *
- * Since: 3.7.0
- **/
-void gnutls_x509_trust_list_set_getissuer_function(
-	gnutls_x509_trust_list_t tlist,
-	gnutls_x509_trust_list_getissuer_function *func)
-{
-	tlist->issuer_callback = func;
-}
-
-/**
- * gnutls_x509_trust_list_set_ptr:
- * @tlist: is a #gnutls_x509_trust_list_t type.
- * @ptr: is the user pointer
- *
- * This function will set (associate) the user given pointer @ptr to
- * the tlist structure. This pointer can be accessed with
- * gnutls_x509_trust_list_get_ptr(). Useful in the callback function
- * gnutls_x509_trust_list_set_getissuer_function.
- *
- * Since: 3.7.0
- **/
-void gnutls_x509_trust_list_set_ptr(gnutls_x509_trust_list_t tlist, void *ptr)
-{
-	tlist->usr_ptr = ptr;
-}
-
-/**
- * gnutls_x509_trust_list_get_ptr:
- * @tlist: is a #gnutls_x509_trust_list_t type.
- *
- * Get user pointer for tlist. Useful in callback function
- * gnutls_x509_trust_list_set_getissuer_function.
- * This is the pointer set with gnutls_x509_trust_list_set_ptr().
- *
- * Returns: the user given pointer from the tlist structure, or
- *   %NULL if it was never set.
- *
- * Since: 3.7.0
- **/
-void *gnutls_x509_trust_list_get_ptr(gnutls_x509_trust_list_t tlist)
-{
-	return tlist->usr_ptr;
-}
-
-/**
- * gnutls_session_set_verify_output_function:
- * @session: is a #gnutls_x509_trust_list_t type.
- * @func: is the callback function
- *
- * This function sets a callback to be called when the peer's certificate
- * chain has to be verified and full path to the trusted root has to be
- * printed.
- *
- * The callback's function prototype is defined in `x509.h':
- * int (*callback)(
- * gnutls_x509_crt_t cert,
- * gnutls_x509_crt_t issuer,
- * gnutls_x509_crl_t crl,
- * unsigned int verification_output);
- *
- * If the callback function is provided then gnutls will call it, in the
- * certificate verification procedure.
- * To verify the certificate chain and print its path uptp the trusted root,
- * functions such as gnutls_certificate_verify_peers(),
- * gnutls_x509_trust_list_verify_crt(), and gnutls_x509_trust_list_verify_crt2()
- * can be used. The callback is set in _gnutls_verify_crt_status() and
- * _gnutls_pkcs11_verify_crt_status().
- *
- * Since: 3.7.0
- **/
-void gnutls_session_set_verify_output_function(
-	gnutls_session_t session, gnutls_verify_output_function *func)
-{
-	session->internals.cert_output_callback = func;
-}
-
 #define TEST_TEXT "test text"
 /* returns error if the certificate has different algorithm than
  * the given key parameters.
  */
 int _gnutls_check_key_cert_match(gnutls_certificate_credentials_t res)
 {
-	gnutls_datum_t test = { (void *)TEST_TEXT, sizeof(TEST_TEXT) - 1 };
-	gnutls_datum_t sig = { NULL, 0 };
+	gnutls_datum_t test = {(void*)TEST_TEXT, sizeof(TEST_TEXT)-1};
+	gnutls_datum_t sig = {NULL, 0};
 	gnutls_digest_algorithm_t dig;
 	int pk, pk2, ret;
 	unsigned sign_algo;
@@ -992,15 +887,16 @@ int _gnutls_check_key_cert_match(gnutls_certificate_credentials_t res)
 	if (res->flags & GNUTLS_CERTIFICATE_SKIP_KEY_CERT_MATCH)
 		return 0;
 
-	pk = gnutls_pubkey_get_pk_algorithm(
-		res->certs[res->ncerts - 1].cert_list[0].pubkey, NULL);
-	pk2 = gnutls_privkey_get_pk_algorithm(res->certs[res->ncerts - 1].pkey,
-					      NULL);
+	pk =
+	    gnutls_pubkey_get_pk_algorithm(res->certs[res->ncerts - 1].
+					   cert_list[0].pubkey, NULL);
+	pk2 =
+	    gnutls_privkey_get_pk_algorithm(res->certs[res->ncerts - 1].pkey,
+					    NULL);
 
 	if (GNUTLS_PK_IS_RSA(pk) && GNUTLS_PK_IS_RSA(pk2)) {
 		if (pk2 == GNUTLS_PK_RSA_PSS && pk == GNUTLS_PK_RSA) {
-			_gnutls_debug_log(
-				"you cannot mix an RSA-PSS key with an RSA certificate\n");
+			_gnutls_debug_log("you cannot mix an RSA-PSS key with an RSA certificate\n");
 			return GNUTLS_E_CERTIFICATE_KEY_MISMATCH;
 		}
 
@@ -1008,9 +904,8 @@ int _gnutls_check_key_cert_match(gnutls_certificate_credentials_t res)
 			pk = GNUTLS_PK_RSA_PSS;
 	} else if (pk2 != pk) {
 		gnutls_assert();
-		_gnutls_debug_log("key is %s, certificate is %s\n",
-				  gnutls_pk_get_name(pk2),
-				  gnutls_pk_get_name(pk));
+		_gnutls_debug_log("key is %s, certificate is %s\n", gnutls_pk_get_name(pk2),
+			gnutls_pk_get_name(pk));
 		return GNUTLS_E_CERTIFICATE_KEY_MISMATCH;
 	}
 
@@ -1029,7 +924,7 @@ int _gnutls_check_key_cert_match(gnutls_certificate_credentials_t res)
 	 * because we cannot always obtain the parameters from the abstract
 	 * keys (e.g. PKCS #11). */
 	ret = gnutls_privkey_sign_data2(res->certs[res->ncerts - 1].pkey,
-					sign_algo, 0, &test, &sig);
+		sign_algo, 0, &test, &sig);
 	if (ret < 0) {
 		/* for some reason we couldn't sign that. That shouldn't have
 		 * happened, but since it did, report the issue and do not
@@ -1038,16 +933,16 @@ int _gnutls_check_key_cert_match(gnutls_certificate_credentials_t res)
 		goto finish;
 	}
 
-	ret = gnutls_pubkey_verify_data2(
-		res->certs[res->ncerts - 1].cert_list[0].pubkey, sign_algo,
-		GNUTLS_VERIFY_ALLOW_BROKEN, &test, &sig);
+	ret = gnutls_pubkey_verify_data2(res->certs[res->ncerts - 1].cert_list[0].pubkey,
+					 sign_algo,
+					 GNUTLS_VERIFY_ALLOW_BROKEN, &test, &sig);
 
 	gnutls_free(sig.data);
 
 	if (ret < 0)
 		return gnutls_assert_val(GNUTLS_E_CERTIFICATE_KEY_MISMATCH);
 
-finish:
+ finish:
 	return 0;
 }
 
@@ -1068,10 +963,11 @@ finish:
  *
  * Since: 3.1.4
  **/
-int gnutls_certificate_verification_status_print(unsigned int status,
-						 gnutls_certificate_type_t type,
-						 gnutls_datum_t *out,
-						 unsigned int flags)
+int
+gnutls_certificate_verification_status_print(unsigned int status,
+					     gnutls_certificate_type_t
+					     type, gnutls_datum_t * out,
+					     unsigned int flags)
 {
 	gnutls_buffer_st str;
 
@@ -1079,90 +975,94 @@ int gnutls_certificate_verification_status_print(unsigned int status,
 
 	if (status == 0)
 		_gnutls_buffer_append_str(&str,
-					  _("The certificate is trusted. "));
+					  _
+					  ("The certificate is trusted. "));
 	else
-		_gnutls_buffer_append_str(
-			&str, _("The certificate is NOT trusted. "));
+		_gnutls_buffer_append_str(&str,
+					  _
+					  ("The certificate is NOT trusted. "));
 
 	if (type == GNUTLS_CRT_X509) {
 		if (status & GNUTLS_CERT_REVOKED)
-			_gnutls_buffer_append_str(
-				&str, _("The certificate chain is revoked. "));
+			_gnutls_buffer_append_str(&str,
+						  _
+						  ("The certificate chain is revoked. "));
 
 		if (status & GNUTLS_CERT_MISMATCH)
-			_gnutls_buffer_append_str(
-				&str,
-				_("The certificate doesn't match the local copy (TOFU). "));
+			_gnutls_buffer_append_str(&str,
+						  _
+						  ("The certificate doesn't match the local copy (TOFU). "));
 
 		if (status & GNUTLS_CERT_REVOCATION_DATA_SUPERSEDED)
-			_gnutls_buffer_append_str(
-				&str,
-				_("The revocation or OCSP data are old and have been superseded. "));
+			_gnutls_buffer_append_str(&str,
+						  _
+						  ("The revocation or OCSP data are old and have been superseded. "));
 
 		if (status & GNUTLS_CERT_REVOCATION_DATA_ISSUED_IN_FUTURE)
-			_gnutls_buffer_append_str(
-				&str,
-				_("The revocation or OCSP data are issued with a future date. "));
+			_gnutls_buffer_append_str(&str,
+						  _
+						  ("The revocation or OCSP data are issued with a future date. "));
 
 		if (status & GNUTLS_CERT_SIGNER_NOT_FOUND)
-			_gnutls_buffer_append_str(
-				&str, _("The certificate issuer is unknown. "));
+			_gnutls_buffer_append_str(&str,
+						  _
+						  ("The certificate issuer is unknown. "));
 
 		if (status & GNUTLS_CERT_SIGNER_NOT_CA)
-			_gnutls_buffer_append_str(
-				&str,
-				_("The certificate issuer is not a CA. "));
+			_gnutls_buffer_append_str(&str,
+						  _
+						  ("The certificate issuer is not a CA. "));
 	}
 
 	if (status & GNUTLS_CERT_INSECURE_ALGORITHM)
-		_gnutls_buffer_append_str(
-			&str,
-			_("The certificate chain uses insecure algorithm. "));
+		_gnutls_buffer_append_str(&str,
+					  _
+					  ("The certificate chain uses insecure algorithm. "));
 
 	if (status & GNUTLS_CERT_SIGNER_CONSTRAINTS_FAILURE)
-		_gnutls_buffer_append_str(
-			&str,
-			_("The certificate chain violates the signer's constraints. "));
+		_gnutls_buffer_append_str(&str,
+					  _
+					  ("The certificate chain violates the signer's constraints. "));
 
 	if (status & GNUTLS_CERT_PURPOSE_MISMATCH)
-		_gnutls_buffer_append_str(
-			&str,
-			_("The certificate chain does not match the intended purpose. "));
+		_gnutls_buffer_append_str(&str,
+					  _
+					  ("The certificate chain does not match the intended purpose. "));
 
 	if (status & GNUTLS_CERT_NOT_ACTIVATED)
-		_gnutls_buffer_append_str(
-			&str,
-			_("The certificate chain uses not yet valid certificate. "));
+		_gnutls_buffer_append_str(&str,
+					  _
+					  ("The certificate chain uses not yet valid certificate. "));
 
 	if (status & GNUTLS_CERT_EXPIRED)
-		_gnutls_buffer_append_str(
-			&str,
-			_("The certificate chain uses expired certificate. "));
+		_gnutls_buffer_append_str(&str,
+					  _
+					  ("The certificate chain uses expired certificate. "));
 
 	if (status & GNUTLS_CERT_SIGNATURE_FAILURE)
-		_gnutls_buffer_append_str(
-			&str,
-			_("The signature in the certificate is invalid. "));
+		_gnutls_buffer_append_str(&str,
+					  _
+					  ("The signature in the certificate is invalid. "));
 
 	if (status & GNUTLS_CERT_UNEXPECTED_OWNER)
-		_gnutls_buffer_append_str(
-			&str,
-			_("The name in the certificate does not match the expected. "));
+		_gnutls_buffer_append_str(&str,
+					  _
+					  ("The name in the certificate does not match the expected. "));
 
 	if (status & GNUTLS_CERT_MISSING_OCSP_STATUS)
-		_gnutls_buffer_append_str(
-			&str,
-			_("The certificate requires the server to include an OCSP status in its response, but the OCSP status is missing. "));
+		_gnutls_buffer_append_str(&str,
+					  _
+					  ("The certificate requires the server to include an OCSP status in its response, but the OCSP status is missing. "));
 
 	if (status & GNUTLS_CERT_INVALID_OCSP_STATUS)
-		_gnutls_buffer_append_str(
-			&str,
-			_("The received OCSP status response is invalid. "));
+		_gnutls_buffer_append_str(&str,
+					  _
+					  ("The received OCSP status response is invalid. "));
 
 	if (status & GNUTLS_CERT_UNKNOWN_CRIT_EXTENSIONS)
-		_gnutls_buffer_append_str(
-			&str,
-			_("The certificate contains an unknown critical extension. "));
+		_gnutls_buffer_append_str(&str,
+					  _
+					  ("The certificate contains an unknown critical extension. "));
 
 	return _gnutls_buffer_to_datum(&str, out, 1);
 }
@@ -1184,8 +1084,9 @@ int gnutls_certificate_verification_status_print(unsigned int status,
  * following RFC7919.
  *
  **/
-void gnutls_certificate_set_dh_params(gnutls_certificate_credentials_t res,
-				      gnutls_dh_params_t dh_params)
+void
+gnutls_certificate_set_dh_params(gnutls_certificate_credentials_t res,
+				 gnutls_dh_params_t dh_params)
 {
 	if (res->deinit_dh_params) {
 		res->deinit_dh_params = 0;
@@ -1194,10 +1095,10 @@ void gnutls_certificate_set_dh_params(gnutls_certificate_credentials_t res,
 	}
 
 	res->dh_params = dh_params;
-	res->dh_sec_param = gnutls_pk_bits_to_sec_param(
-		GNUTLS_PK_DH, _gnutls_mpi_get_nbits(dh_params->params[0]));
+	res->dh_sec_param = gnutls_pk_bits_to_sec_param(GNUTLS_PK_DH, _gnutls_mpi_get_nbits(dh_params->params[0]));
 }
 
+
 /**
  * gnutls_certificate_set_known_dh_params:
  * @res: is a gnutls_certificate_credentials_t type
@@ -1217,12 +1118,13 @@ void gnutls_certificate_set_dh_params(gnutls_certificate_credentials_t res,
  *
  * Since: 3.5.6
  **/
-int gnutls_certificate_set_known_dh_params(gnutls_certificate_credentials_t res,
-					   gnutls_sec_param_t sec_param)
+int
+gnutls_certificate_set_known_dh_params(gnutls_certificate_credentials_t res,
+				       gnutls_sec_param_t sec_param)
 {
 	res->dh_sec_param = sec_param;
 
 	return 0;
 }
 
-#endif /* DH */
+#endif				/* DH */
