@@ -20,38 +20,37 @@
  *
  */
 
-#ifndef GNUTLS_LIB_CONSTATE_H
-#define GNUTLS_LIB_CONSTATE_H
+#ifndef GNUTLS_CONSTATE_H
+#define GNUTLS_CONSTATE_H
 
 int _gnutls_set_cipher_suite2(gnutls_session_t session,
-			      const gnutls_cipher_suite_entry_st *cs);
+			     const gnutls_cipher_suite_entry_st *cs);
 
-int _gnutls_epoch_set_keys(gnutls_session_t session, uint16_t epoch,
-			   hs_stage_t stage);
+int _gnutls_epoch_set_keys(gnutls_session_t session, uint16_t epoch, hs_stage_t stage);
 int _gnutls_connection_state_init(gnutls_session_t session);
 int _gnutls_read_connection_state_init(gnutls_session_t session);
 int _gnutls_write_connection_state_init(gnutls_session_t session);
 
-#define _gnutls_epoch_bump(session) (session)->security_parameters.epoch_next++
+#define _gnutls_epoch_bump(session) \
+	(session)->security_parameters.epoch_next++
 
 int _gnutls_epoch_dup(gnutls_session_t session, unsigned int epoch_rel);
 
 int _gnutls_epoch_get(gnutls_session_t session, unsigned int epoch_rel,
-		      record_parameters_st **params_out);
-int _gnutls_epoch_setup_next(gnutls_session_t session, unsigned null_epoch,
-			     record_parameters_st **newp);
+		      record_parameters_st ** params_out);
+int _gnutls_epoch_setup_next(gnutls_session_t session, unsigned null_epoch, record_parameters_st **newp);
 void _gnutls_epoch_gc(gnutls_session_t session);
-void _gnutls_epoch_free(gnutls_session_t session, record_parameters_st *state);
+void _gnutls_epoch_free(gnutls_session_t session,
+			record_parameters_st * state);
 
 void _gnutls_set_resumed_parameters(gnutls_session_t session);
 
 int _tls13_connection_state_init(gnutls_session_t session, hs_stage_t stage);
-int _tls13_read_connection_state_init(gnutls_session_t session,
-				      hs_stage_t stage);
-int _tls13_write_connection_state_init(gnutls_session_t session,
-				       hs_stage_t stage);
+int _tls13_read_connection_state_init(gnutls_session_t session, hs_stage_t stage);
+int _tls13_write_connection_state_init(gnutls_session_t session, hs_stage_t stage);
 
-static inline int _gnutls_epoch_is_valid(gnutls_session_t session, int epoch)
+static inline int _gnutls_epoch_is_valid(gnutls_session_t session,
+					 int epoch)
 {
 	record_parameters_st *params;
 	int ret;
@@ -63,6 +62,7 @@ static inline int _gnutls_epoch_is_valid(gnutls_session_t session, int epoch)
 	return 1;
 }
 
+
 static inline int _gnutls_epoch_refcount_inc(gnutls_session_t session,
 					     int epoch)
 {
@@ -95,4 +95,4 @@ static inline int _gnutls_epoch_refcount_dec(gnutls_session_t session,
 	return 0;
 }
 
-#endif /* GNUTLS_LIB_CONSTATE_H */
+#endif
