@@ -20,15 +20,11 @@
  *
  */
 
-#ifndef GNUTLS_LIB_DB_H
-#define GNUTLS_LIB_DB_H
-
 int _gnutls_server_register_current_session(gnutls_session_t session);
 int _gnutls_server_restore_session(gnutls_session_t session,
-				   uint8_t *session_id, int session_id_size);
+				   uint8_t * session_id,
+				   int session_id_size);
 
 int _gnutls_check_resumed_params(gnutls_session_t session);
 
-#define PACKED_SESSION_MAGIC ((0xfadebadd) + (_gnutls_global_version))
-
-#endif /* GNUTLS_LIB_DB_H */
+#define PACKED_SESSION_MAGIC ((0xfadebadd)+(GNUTLS_VERSION_NUMBER))
