@@ -24,7 +24,7 @@
 #include "gnutls_int.h"
 #include "errors.h"
 #include <datum.h>
-#include <x509_b64.h> /* for PKCS3 PEM decoding */
+#include <x509_b64.h>		/* for PKCS3 PEM decoding */
 #include <global.h>
 #include <dh.h>
 #include <pk.h>
@@ -35,8 +35,9 @@
 #include "debug.h"
 #include "state.h"
 
-static int set_dh_pk_params(gnutls_session_t session, bigint_t g, bigint_t p,
-			    bigint_t q, unsigned q_bits)
+static
+int set_dh_pk_params(gnutls_session_t session, bigint_t g, bigint_t p,
+			unsigned q_bits)
 {
 	/* just in case we are resuming a session */
 	gnutls_pk_params_release(&session->key.proto.tls12.dh.params);
@@ -49,24 +50,11 @@ static int set_dh_pk_params(gnutls_session_t session, bigint_t g, bigint_t p,
 
 	session->key.proto.tls12.dh.params.params[DH_P] = _gnutls_mpi_copy(p);
 	if (session->key.proto.tls12.dh.params.params[DH_P] == NULL) {
-		_gnutls_mpi_release(
-			&session->key.proto.tls12.dh.params.params[DH_G]);
+		_gnutls_mpi_release(&session->key.proto.tls12.dh.params.params[DH_G]);
 		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 	}
 
-	if (q) {
-		session->key.proto.tls12.dh.params.params[DH_Q] =
-			_gnutls_mpi_copy(q);
-		if (session->key.proto.tls12.dh.params.params[DH_Q] == NULL) {
-			_gnutls_mpi_release(&session->key.proto.tls12.dh.params
-						     .params[DH_P]);
-			_gnutls_mpi_release(&session->key.proto.tls12.dh.params
-						     .params[DH_G]);
-			return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
-		}
-	}
-	/* include, possibly empty, q */
-	session->key.proto.tls12.dh.params.params_nr = 3;
+	session->key.proto.tls12.dh.params.params_nr = 3; /* include empty q */
 	session->key.proto.tls12.dh.params.algo = GNUTLS_PK_DH;
 	session->key.proto.tls12.dh.params.qbits = q_bits;
 
@@ -77,13 +65,12 @@ static int set_dh_pk_params(gnutls_session_t session, bigint_t g, bigint_t p,
  * that being the negotiated RFC7919 group, the callback, and the
  * provided parameters structure.
  */
-int _gnutls_figure_dh_params(gnutls_session_t session,
-			     gnutls_dh_params_t dh_params,
-			     gnutls_params_function *func,
-			     gnutls_sec_param_t sec_param)
+int
+_gnutls_figure_dh_params(gnutls_session_t session, gnutls_dh_params_t dh_params,
+		      gnutls_params_function * func, gnutls_sec_param_t sec_param)
 {
 	gnutls_params_st params;
-	bigint_t p, g, q = NULL;
+	bigint_t p, g;
 	unsigned free_pg = 0;
 	int ret;
 	unsigned q_bits = 0, i;
@@ -95,53 +82,26 @@ int _gnutls_figure_dh_params(gnutls_session_t session,
 
 	/* if we negotiated RFC7919 FFDHE */
 	if (group && group->pk == GNUTLS_PK_DH) {
-		for (i = 0; i < session->internals.priorities->groups.size;
-		     i++) {
-			if (session->internals.priorities->groups.entry[i] ==
-			    group) {
-				ret = _gnutls_mpi_init_scan_nz(
-					&p,
-					session->internals.priorities->groups
-						.entry[i]
-						->prime->data,
-					session->internals.priorities->groups
-						.entry[i]
-						->prime->size);
+		for (i=0;i<session->internals.priorities->groups.size;i++) {
+			if (session->internals.priorities->groups.entry[i] == group) {
+				ret = _gnutls_mpi_init_scan_nz(&p,
+						session->internals.priorities->groups.entry[i]->prime->data,
+						session->internals.priorities->groups.entry[i]->prime->size);
 				if (ret < 0)
 					return gnutls_assert_val(ret);
 
 				free_pg = 1;
 
-				ret = _gnutls_mpi_init_scan_nz(
-					&g,
-					session->internals.priorities->groups
-						.entry[i]
-						->generator->data,
-					session->internals.priorities->groups
-						.entry[i]
-						->generator->size);
-				if (ret < 0) {
-					gnutls_assert();
-					goto cleanup;
-				}
-
-				ret = _gnutls_mpi_init_scan_nz(
-					&q,
-					session->internals.priorities->groups
-						.entry[i]
-						->q->data,
-					session->internals.priorities->groups
-						.entry[i]
-						->q->size);
+				ret = _gnutls_mpi_init_scan_nz(&g,
+						session->internals.priorities->groups.entry[i]->generator->data,
+						session->internals.priorities->groups.entry[i]->generator->size);
 				if (ret < 0) {
 					gnutls_assert();
 					goto cleanup;
 				}
 
 				session->internals.hsk_flags |= HSK_USED_FFDHE;
-				q_bits = *session->internals.priorities->groups
-						  .entry[i]
-						  ->q_bits;
+				q_bits = *session->internals.priorities->groups.entry[i]->q_bits;
 				goto finished;
 			}
 		}
@@ -150,51 +110,34 @@ int _gnutls_figure_dh_params(gnutls_session_t session,
 		 * as we received that extension */
 		return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 	} else if (sec_param) {
-		unsigned bits =
-			gnutls_sec_param_to_pk_bits(GNUTLS_PK_DH, sec_param) /
-			8;
+		unsigned bits = gnutls_sec_param_to_pk_bits(GNUTLS_PK_DH, sec_param)/8;
 
-		for (i = 0; i < session->internals.priorities->groups.size;
-		     i++) {
-			if (!session->internals.priorities->groups.entry[i]
-				     ->prime)
+		for (i=0;i<session->internals.priorities->groups.size;i++) {
+			if (!session->internals.priorities->groups.entry[i]->prime)
 				continue;
 
-			if (bits <=
-			    session->internals.priorities->groups.entry[i]
-				    ->prime->size) {
-				ret = _gnutls_mpi_init_scan_nz(
-					&p,
-					session->internals.priorities->groups
-						.entry[i]
-						->prime->data,
-					session->internals.priorities->groups
-						.entry[i]
-						->prime->size);
+			if (bits <= session->internals.priorities->groups.entry[i]->prime->size) {
+				ret = _gnutls_mpi_init_scan_nz(&p,
+						session->internals.priorities->groups.entry[i]->prime->data,
+						session->internals.priorities->groups.entry[i]->prime->size);
 				if (ret < 0)
 					return gnutls_assert_val(ret);
 
 				free_pg = 1;
 
-				ret = _gnutls_mpi_init_scan_nz(
-					&g,
-					session->internals.priorities->groups
-						.entry[i]
-						->generator->data,
-					session->internals.priorities->groups
-						.entry[i]
-						->generator->size);
+				ret = _gnutls_mpi_init_scan_nz(&g,
+						session->internals.priorities->groups.entry[i]->generator->data,
+						session->internals.priorities->groups.entry[i]->generator->size);
 				if (ret < 0) {
 					gnutls_assert();
 					goto cleanup;
 				}
 
-				q_bits = *session->internals.priorities->groups
-						  .entry[i]
-						  ->q_bits;
+				q_bits = *session->internals.priorities->groups.entry[i]->q_bits;
 				goto finished;
 			}
 		}
+
 	}
 
 	if (dh_params) {
@@ -208,29 +151,28 @@ int _gnutls_figure_dh_params(gnutls_session_t session,
 			g = params.params.dh->params[1];
 			q_bits = params.params.dh->q_bits;
 		} else
-			return gnutls_assert_val(
-				GNUTLS_E_NO_TEMPORARY_DH_PARAMS);
+			return gnutls_assert_val(GNUTLS_E_NO_TEMPORARY_DH_PARAMS);
 	} else
 		return gnutls_assert_val(GNUTLS_E_NO_TEMPORARY_DH_PARAMS);
 
-finished:
+ finished:
 	_gnutls_dh_save_group(session, g, p);
 
-	ret = set_dh_pk_params(session, g, p, q, q_bits);
+	ret = set_dh_pk_params(session, g, p, q_bits);
 	if (ret < 0) {
 		gnutls_assert();
 	}
 
-cleanup:
+ cleanup:
 	if (free_pg) {
 		_gnutls_mpi_release(&p);
-		_gnutls_mpi_release(&q);
 		_gnutls_mpi_release(&g);
 	}
 	if (params.deinit && params.type == GNUTLS_PARAMS_DH)
 		gnutls_dh_params_deinit(params.params.dh);
 
 	return ret;
+
 }
 
 /* returns the prime and the generator of DH params.
@@ -245,6 +187,7 @@ const bigint_t *_gnutls_dh_params_to_mpi(gnutls_dh_params_t dh_primes)
 	return dh_primes->params;
 }
 
+
 /**
  * gnutls_dh_params_import_raw:
  * @dh_params: The parameters
@@ -258,9 +201,10 @@ const bigint_t *_gnutls_dh_params_to_mpi(gnutls_dh_params_t dh_primes)
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned,
  *   otherwise a negative error code is returned.
  **/
-int gnutls_dh_params_import_raw(gnutls_dh_params_t dh_params,
-				const gnutls_datum_t *prime,
-				const gnutls_datum_t *generator)
+int
+gnutls_dh_params_import_raw(gnutls_dh_params_t dh_params,
+			    const gnutls_datum_t * prime,
+			    const gnutls_datum_t * generator)
 {
 	return gnutls_dh_params_import_raw2(dh_params, prime, generator, 0);
 }
@@ -276,18 +220,29 @@ int gnutls_dh_params_import_raw(gnutls_dh_params_t dh_params,
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned,
  *   otherwise a negative error code is returned.
  **/
-int gnutls_dh_params_import_dsa(gnutls_dh_params_t dh_params,
-				gnutls_x509_privkey_t key)
+int
+gnutls_dh_params_import_dsa(gnutls_dh_params_t dh_params, gnutls_x509_privkey_t key)
 {
 	gnutls_datum_t p, g, q;
+	bigint_t tmp_q;
 	int ret;
 
 	ret = gnutls_x509_privkey_export_dsa_raw(key, &p, &q, &g, NULL, NULL);
 	if (ret < 0)
 		return gnutls_assert_val(ret);
 
-	ret = gnutls_dh_params_import_raw3(dh_params, &p, &q, &g);
+	ret = _gnutls_mpi_init_scan_nz(&tmp_q, q.data, q.size);
+	if (ret < 0) {
+		gnutls_assert();
+		ret = GNUTLS_E_MPI_SCAN_FAILED;
+		goto cleanup;
+	}
 
+	ret = gnutls_dh_params_import_raw2(dh_params, &p, &g, _gnutls_mpi_get_nbits(tmp_q));
+
+	_gnutls_mpi_release(&tmp_q);
+
+ cleanup:
 	gnutls_free(p.data);
 	gnutls_free(g.data);
 	gnutls_free(q.data);
@@ -309,10 +264,11 @@ int gnutls_dh_params_import_dsa(gnutls_dh_params_t dh_params,
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned,
  *   otherwise a negative error code is returned.
  **/
-int gnutls_dh_params_import_raw2(gnutls_dh_params_t dh_params,
-				 const gnutls_datum_t *prime,
-				 const gnutls_datum_t *generator,
-				 unsigned key_bits)
+int
+gnutls_dh_params_import_raw2(gnutls_dh_params_t dh_params,
+			     const gnutls_datum_t * prime,
+			     const gnutls_datum_t * generator,
+			     unsigned key_bits)
 {
 	bigint_t tmp_prime, tmp_g;
 	size_t siz;
@@ -339,63 +295,6 @@ int gnutls_dh_params_import_raw2(gnutls_dh_params_t dh_params,
 	return 0;
 }
 
-/**
- * gnutls_dh_params_import_raw3:
- * @dh_params: The parameters
- * @prime: holds the new prime
- * @q: holds the subgroup if available, otherwise NULL
- * @generator: holds the new generator
- *
- * This function will replace the pair of prime and generator for use
- * in the Diffie-Hellman key exchange.  The new parameters should be
- * stored in the appropriate gnutls_datum.
- *
- * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned,
- *   otherwise a negative error code is returned.
- **/
-int gnutls_dh_params_import_raw3(gnutls_dh_params_t dh_params,
-				 const gnutls_datum_t *prime,
-				 const gnutls_datum_t *q,
-				 const gnutls_datum_t *generator)
-{
-	bigint_t tmp_p, tmp_g, tmp_q = NULL;
-
-	if (_gnutls_mpi_init_scan_nz(&tmp_p, prime->data, prime->size)) {
-		gnutls_assert();
-		return GNUTLS_E_MPI_SCAN_FAILED;
-	}
-
-	if (_gnutls_mpi_init_scan_nz(&tmp_g, generator->data,
-				     generator->size)) {
-		_gnutls_mpi_release(&tmp_p);
-		gnutls_assert();
-		return GNUTLS_E_MPI_SCAN_FAILED;
-	}
-
-	if (q) {
-		if (_gnutls_mpi_init_scan_nz(&tmp_q, q->data, q->size)) {
-			_gnutls_mpi_release(&tmp_p);
-			_gnutls_mpi_release(&tmp_g);
-			gnutls_assert();
-			return GNUTLS_E_MPI_SCAN_FAILED;
-		}
-	} else if (_gnutls_fips_mode_enabled()) {
-		/* Mandatory in FIPS mode */
-		gnutls_assert();
-		return GNUTLS_E_DH_PRIME_UNACCEPTABLE;
-	}
-
-	/* store the generated values
-	 */
-	dh_params->params[0] = tmp_p;
-	dh_params->params[1] = tmp_g;
-	dh_params->params[2] = tmp_q;
-	if (tmp_q)
-		dh_params->q_bits = _gnutls_mpi_get_nbits(tmp_q);
-
-	return 0;
-}
-
 /**
  * gnutls_dh_params_init:
  * @dh_params: The parameters
@@ -405,8 +304,9 @@ int gnutls_dh_params_import_raw3(gnutls_dh_params_t dh_params,
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned,
  *   otherwise a negative error code is returned.
  **/
-int gnutls_dh_params_init(gnutls_dh_params_t *dh_params)
+int gnutls_dh_params_init(gnutls_dh_params_t * dh_params)
 {
+
 	(*dh_params) = gnutls_calloc(1, sizeof(dh_params_st));
 	if (*dh_params == NULL) {
 		gnutls_assert();
@@ -414,6 +314,7 @@ int gnutls_dh_params_init(gnutls_dh_params_t *dh_params)
 	}
 
 	return 0;
+
 }
 
 /**
@@ -429,9 +330,9 @@ void gnutls_dh_params_deinit(gnutls_dh_params_t dh_params)
 
 	_gnutls_mpi_release(&dh_params->params[0]);
 	_gnutls_mpi_release(&dh_params->params[1]);
-	_gnutls_mpi_release(&dh_params->params[2]);
 
 	gnutls_free(dh_params);
+
 }
 
 /**
@@ -452,8 +353,6 @@ int gnutls_dh_params_cpy(gnutls_dh_params_t dst, gnutls_dh_params_t src)
 
 	dst->params[0] = _gnutls_mpi_copy(src->params[0]);
 	dst->params[1] = _gnutls_mpi_copy(src->params[1]);
-	if (src->params[2])
-		dst->params[2] = _gnutls_mpi_copy(src->params[2]);
 	dst->q_bits = src->q_bits;
 
 	if (dst->params[0] == NULL || dst->params[1] == NULL)
@@ -462,6 +361,7 @@ int gnutls_dh_params_cpy(gnutls_dh_params_t dst, gnutls_dh_params_t src)
 	return 0;
 }
 
+
 /**
  * gnutls_dh_params_generate2:
  * @dparams: The parameters
@@ -489,7 +389,8 @@ int gnutls_dh_params_cpy(gnutls_dh_params_t dst, gnutls_dh_params_t src)
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned,
  *   otherwise a negative error code is returned.
  **/
-int gnutls_dh_params_generate2(gnutls_dh_params_t dparams, unsigned int bits)
+int
+gnutls_dh_params_generate2(gnutls_dh_params_t dparams, unsigned int bits)
 {
 	int ret;
 	gnutls_pk_params_st params;
@@ -524,19 +425,22 @@ int gnutls_dh_params_generate2(gnutls_dh_params_t dparams, unsigned int bits)
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned,
  *   otherwise a negative error code is returned.
  **/
-int gnutls_dh_params_import_pkcs3(gnutls_dh_params_t params,
-				  const gnutls_datum_t *pkcs3_params,
-				  gnutls_x509_crt_fmt_t format)
+int
+gnutls_dh_params_import_pkcs3(gnutls_dh_params_t params,
+			      const gnutls_datum_t * pkcs3_params,
+			      gnutls_x509_crt_fmt_t format)
 {
-	asn1_node c2;
+	ASN1_TYPE c2;
 	int result, need_free = 0;
 	unsigned int q_bits;
 	gnutls_datum_t _params;
 
 	if (format == GNUTLS_X509_FMT_PEM) {
+
 		result = _gnutls_fbase64_decode("DH PARAMETERS",
 						pkcs3_params->data,
-						pkcs3_params->size, &_params);
+						pkcs3_params->size,
+						&_params);
 
 		if (result < 0) {
 			gnutls_assert();
@@ -549,9 +453,9 @@ int gnutls_dh_params_import_pkcs3(gnutls_dh_params_t params,
 		_params.size = pkcs3_params->size;
 	}
 
-	if ((result = asn1_create_element(_gnutls_get_gnutls_asn(),
-					  "GNUTLS.DHParameter", &c2)) !=
-	    ASN1_SUCCESS) {
+	if ((result = asn1_create_element
+	     (_gnutls_get_gnutls_asn(), "GNUTLS.DHParameter", &c2))
+	    != ASN1_SUCCESS) {
 		gnutls_assert();
 		if (need_free != 0) {
 			gnutls_free(_params.data);
@@ -639,19 +543,20 @@ int gnutls_dh_params_import_pkcs3(gnutls_dh_params_t params,
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned,
  *   otherwise a negative error code is returned.
  **/
-int gnutls_dh_params_export_pkcs3(gnutls_dh_params_t params,
-				  gnutls_x509_crt_fmt_t format,
-				  unsigned char *params_data,
-				  size_t *params_data_size)
+int
+gnutls_dh_params_export_pkcs3(gnutls_dh_params_t params,
+			      gnutls_x509_crt_fmt_t format,
+			      unsigned char *params_data,
+			      size_t * params_data_size)
 {
-	gnutls_datum_t out = { NULL, 0 };
+	gnutls_datum_t out = {NULL, 0};
 	int ret;
 
 	ret = gnutls_dh_params_export2_pkcs3(params, format, &out);
 	if (ret < 0)
 		return gnutls_assert_val(ret);
 
-	if (*params_data_size < (unsigned)out.size + 1) {
+	if (*params_data_size < (unsigned) out.size + 1) {
 		gnutls_assert();
 		gnutls_free(out.data);
 		*params_data_size = out.size + 1;
@@ -688,11 +593,12 @@ int gnutls_dh_params_export_pkcs3(gnutls_dh_params_t params,
  *
  * Since: 3.1.3
  **/
-int gnutls_dh_params_export2_pkcs3(gnutls_dh_params_t params,
-				   gnutls_x509_crt_fmt_t format,
-				   gnutls_datum_t *out)
+int
+gnutls_dh_params_export2_pkcs3(gnutls_dh_params_t params,
+			       gnutls_x509_crt_fmt_t format,
+			       gnutls_datum_t * out)
 {
-	asn1_node c2;
+	ASN1_TYPE c2;
 	int result;
 	size_t g_size, p_size;
 	uint8_t *p_data, *g_data;
@@ -713,12 +619,13 @@ int gnutls_dh_params_export2_pkcs3(gnutls_dh_params_t params,
 	g_data = &all_data[p_size];
 	_gnutls_mpi_print_lz(params->params[1], g_data, &g_size);
 
+
 	/* Ok. Now we have the data. Create the asn1 structures
 	 */
 
-	if ((result = asn1_create_element(_gnutls_get_gnutls_asn(),
-					  "GNUTLS.DHParameter", &c2)) !=
-	    ASN1_SUCCESS) {
+	if ((result = asn1_create_element
+	     (_gnutls_get_gnutls_asn(), "GNUTLS.DHParameter", &c2))
+	    != ASN1_SUCCESS) {
 		gnutls_assert();
 		gnutls_free(all_data);
 		return _gnutls_asn2err(result);
@@ -726,8 +633,8 @@ int gnutls_dh_params_export2_pkcs3(gnutls_dh_params_t params,
 
 	/* Write PRIME 
 	 */
-	if ((result = asn1_write_value(c2, "prime", p_data, p_size)) !=
-	    ASN1_SUCCESS) {
+	if ((result = asn1_write_value(c2, "prime",
+				       p_data, p_size)) != ASN1_SUCCESS) {
 		gnutls_assert();
 		gnutls_free(all_data);
 		asn1_delete_structure(&c2);
@@ -735,10 +642,12 @@ int gnutls_dh_params_export2_pkcs3(gnutls_dh_params_t params,
 	}
 
 	if (params->q_bits > 0)
-		result = _gnutls_x509_write_uint32(c2, "privateValueLength",
-						   params->q_bits);
+		result =
+		    _gnutls_x509_write_uint32(c2, "privateValueLength",
+					      params->q_bits);
 	else
-		result = asn1_write_value(c2, "privateValueLength", NULL, 0);
+		result =
+		    asn1_write_value(c2, "privateValueLength", NULL, 0);
 
 	if (result < 0) {
 		gnutls_assert();
@@ -749,8 +658,8 @@ int gnutls_dh_params_export2_pkcs3(gnutls_dh_params_t params,
 
 	/* Write the GENERATOR
 	 */
-	if ((result = asn1_write_value(c2, "base", g_data, g_size)) !=
-	    ASN1_SUCCESS) {
+	if ((result = asn1_write_value(c2, "base",
+				       g_data, g_size)) != ASN1_SUCCESS) {
 		gnutls_assert();
 		gnutls_free(all_data);
 		asn1_delete_structure(&c2);
@@ -759,6 +668,7 @@ int gnutls_dh_params_export2_pkcs3(gnutls_dh_params_t params,
 
 	gnutls_free(all_data);
 
+
 	if (format == GNUTLS_X509_FMT_DER) {
 		result = _gnutls_x509_der_encode(c2, "", out, 0);
 
@@ -767,7 +677,7 @@ int gnutls_dh_params_export2_pkcs3(gnutls_dh_params_t params,
 		if (result < 0)
 			return gnutls_assert_val(result);
 
-	} else { /* PEM */
+	} else {		/* PEM */
 		gnutls_datum_t t;
 
 		result = _gnutls_x509_der_encode(c2, "", &t, 0);
@@ -777,8 +687,9 @@ int gnutls_dh_params_export2_pkcs3(gnutls_dh_params_t params,
 		if (result < 0)
 			return gnutls_assert_val(result);
 
-		result = _gnutls_fbase64_encode("DH PARAMETERS", t.data, t.size,
-						out);
+		result =
+		    _gnutls_fbase64_encode("DH PARAMETERS", t.data, t.size,
+					   out);
 
 		gnutls_free(t.data);
 
@@ -806,9 +717,10 @@ int gnutls_dh_params_export2_pkcs3(gnutls_dh_params_t params,
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned,
  *   otherwise a negative error code is returned.
  **/
-int gnutls_dh_params_export_raw(gnutls_dh_params_t params,
-				gnutls_datum_t *prime,
-				gnutls_datum_t *generator, unsigned int *bits)
+int
+gnutls_dh_params_export_raw(gnutls_dh_params_t params,
+			    gnutls_datum_t * prime,
+			    gnutls_datum_t * generator, unsigned int *bits)
 {
 	int ret;
 
@@ -834,4 +746,5 @@ int gnutls_dh_params_export_raw(gnutls_dh_params_t params,
 		*bits = params->q_bits;
 
 	return 0;
+
 }
