@@ -17,13 +17,15 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  *
  */
 
 #include "gnutls_int.h"
 #include "auth/psk.h"
+#include "handshake.h"
 #include "secrets.h"
+#include "tls13/anti_replay.h"
 #include "tls13/psk_ext_parser.h"
 #include "tls13/finished.h"
 #include "tls13/session_ticket.h"
@@ -33,10 +35,9 @@
 #include <assert.h>
 
 static int
-compute_psk_from_ticket(const tls13_ticket_t *ticket, gnutls_datum_t *key)
+compute_psk_from_ticket(const tls13_ticket_st *ticket, gnutls_datum_t *key)
 {
 	int ret;
-	char label[] = "resumption";
 
 	if (unlikely(ticket->prf == NULL || ticket->prf->output_size == 0))
 		return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
@@ -49,7 +50,7 @@ compute_psk_from_ticket(const tls13_ticket_t *ticket, gnutls_datum_t *key)
 	key->size = ticket->prf->output_size;
 
 	ret = _tls13_expand_secret2(ticket->prf,
-				    label, sizeof(label)-1,
+				    RESUMPTION_LABEL, sizeof(RESUMPTION_LABEL)-1,
 				    ticket->nonce, ticket->nonce_size,
 				    ticket->resumption_master_secret,
 				    key->size,
@@ -67,9 +68,9 @@ compute_binder_key(const mac_entry_st *prf,
 		   void *out)
 {
 	int ret;
-	const char ext_label[] = "ext binder";
+	const char ext_label[] = EXT_BINDER_LABEL;
 	const size_t ext_label_len = sizeof(ext_label) - 1;
-	const char res_label[] = "res binder";
+	const char res_label[] = RES_BINDER_LABEL;
 	const size_t res_label_len = sizeof(res_label) - 1;
 	const char *label = resuming ? res_label : ext_label;
 	size_t label_len = resuming ? res_label_len : ext_label_len;
@@ -201,7 +202,7 @@ client_send_params(gnutls_session_t session,
 	unsigned next_idx;
 	const mac_entry_st *prf_res = NULL;
 	const mac_entry_st *prf_psk = NULL;
-	time_t cur_time;
+	struct timespec cur_time;
 	uint32_t ticket_age, ob_ticket_age;
 	int free_username = 0;
 	psk_auth_info_t info = NULL;
@@ -235,16 +236,21 @@ client_send_params(gnutls_session_t session,
 
 		prf_res = session->internals.tls13_ticket.prf;
 
-		cur_time = gnutls_time(0);
-		if (unlikely(cur_time < session->internals.tls13_ticket.timestamp)) {
+		gnutls_gettime(&cur_time);
+		if (unlikely(_gnutls_timespec_cmp(&cur_time,
+						  &session->internals.
+						  tls13_ticket.
+						  arrival_time) < 0)) {
 			gnutls_assert();
 			_gnutls13_session_ticket_unset(session);
 			goto ignore_ticket;
 		}
 
 		/* Check whether the ticket is stale */
-		ticket_age = cur_time - session->internals.tls13_ticket.timestamp;
-		if (ticket_age > session->internals.tls13_ticket.lifetime) {
+		ticket_age = timespec_sub_ms(&cur_time,
+					     &session->internals.tls13_ticket.
+					     arrival_time);
+		if (ticket_age / 1000 > session->internals.tls13_ticket.lifetime) {
 			_gnutls13_session_ticket_unset(session);
 			goto ignore_ticket;
 		}
@@ -256,7 +262,7 @@ client_send_params(gnutls_session_t session,
 		}
 
 		/* Calculate obfuscated ticket age, in milliseconds, mod 2^32 */
-		ob_ticket_age = ticket_age * 1000 + session->internals.tls13_ticket.age_add;
+		ob_ticket_age = ticket_age + session->internals.tls13_ticket.age_add;
 
 		if ((ret = _gnutls_buffer_append_data_prefix(extdata, 16,
 							     session->internals.tls13_ticket.ticket.data,
@@ -310,7 +316,7 @@ client_send_params(gnutls_session_t session,
 			user_key.size = tkey.size;
 		}
 
-		ret = _gnutls_auth_info_set(session, GNUTLS_CRD_PSK, sizeof(psk_auth_info_st), 1);
+		ret = _gnutls_auth_info_init(session, GNUTLS_CRD_PSK, sizeof(psk_auth_info_st), 1);
 		if (ret < 0) {
 			gnutls_assert();
 			goto cleanup;
@@ -476,8 +482,10 @@ static int server_recv_params(gnutls_session_t session,
 	psk_ext_iter_st psk_iter;
 	struct psk_st psk;
 	psk_auth_info_t info;
-	tls13_ticket_t ticket_data;
-	uint32_t ticket_age;
+	tls13_ticket_st ticket_data;
+	/* These values should be set properly when session ticket is accepted. */
+	uint32_t ticket_age = UINT32_MAX;
+	struct timespec ticket_creation_time = { 0, 0 };
 	bool resuming;
 
 	ret = _gnutls13_psk_ext_parser_init(&psk_parser, data, len);
@@ -507,12 +515,6 @@ static int server_recv_params(gnutls_session_t session,
 			session->internals.resumption_requested = 1;
 
 			/* Check whether ticket is stale or not */
-			if (psk.ob_ticket_age < ticket_data.age_add) {
-				gnutls_assert();
-				tls13_ticket_deinit(&ticket_data);
-				continue;
-			}
-
 			ticket_age = psk.ob_ticket_age - ticket_data.age_add;
 			if (ticket_age / 1000 > ticket_data.lifetime) {
 				gnutls_assert();
@@ -527,6 +529,10 @@ static int server_recv_params(gnutls_session_t session,
 				continue;
 			}
 
+			memcpy(&ticket_creation_time,
+			       &ticket_data.creation_time,
+			       sizeof(struct timespec));
+
 			tls13_ticket_deinit(&ticket_data);
 
 			resuming = 1;
@@ -535,7 +541,7 @@ static int server_recv_params(gnutls_session_t session,
 			   psk.ob_ticket_age == 0 &&
 			   psk.identity.size > 0 && psk.identity.size <= MAX_USERNAME_SIZE) {
 			/* _gnutls_psk_pwd_find_entry() expects 0-terminated identities */
-			char identity_str[psk.identity.size + 1];
+			char identity_str[MAX_USERNAME_SIZE + 1];
 
 			prf = pskcred->binder_algo;
 
@@ -600,7 +606,7 @@ static int server_recv_params(gnutls_session_t session,
 	if (!resuming) {
 		assert(psk.identity.size < sizeof(info->username));
 
-		ret = _gnutls_auth_info_set(session, GNUTLS_CRD_PSK, sizeof(psk_auth_info_st), 1);
+		ret = _gnutls_auth_info_init(session, GNUTLS_CRD_PSK, sizeof(psk_auth_info_st), 1);
 		if (ret < 0) {
 			gnutls_assert();
 			goto fail;
@@ -613,6 +619,24 @@ static int server_recv_params(gnutls_session_t session,
 		info->username[psk.identity.size] = 0;
 		_gnutls_handshake_log("EXT[%p]: selected PSK identity: %s (%d)\n", session, info->username, psk_index);
 	} else {
+		if (session->internals.hsk_flags & HSK_EARLY_DATA_ACCEPTED) {
+			if (session->internals.anti_replay) {
+				ret = _gnutls_anti_replay_check(session->internals.anti_replay,
+								ticket_age,
+								&ticket_creation_time,
+								&binder_recvd);
+				if (ret < 0) {
+					session->internals.hsk_flags &= ~HSK_EARLY_DATA_ACCEPTED;
+					_gnutls_handshake_log("EXT[%p]: replay detected; rejecting early data\n",
+						      session);
+				}
+			} else {
+				_gnutls_handshake_log("EXT[%p]: anti-replay is not enabled; rejecting early data\n",
+						      session);
+				session->internals.hsk_flags &= ~HSK_EARLY_DATA_ACCEPTED;
+			}
+		}
+
 		session->internals.resumed = RESUME_TRUE;
 		_gnutls_handshake_log("EXT[%p]: selected resumption PSK identity (%d)\n", session, psk_index);
 	}
