@@ -20,11 +20,11 @@
  *
  */
 
-#ifndef GNUTLS_LIB_EXT_RECORD_SIZE_LIMIT_H
-#define GNUTLS_LIB_EXT_RECORD_SIZE_LIMIT_H
+#ifndef EXT_RECORD_SIZE_LIMIT_H
+#define EXT_RECORD_SIZE_LIMIT_H
 
 #include <hello_ext.h>
 
 extern const hello_ext_entry_st ext_mod_record_size_limit;
 
-#endif /* GNUTLS_LIB_EXT_RECORD_SIZE_LIMIT_H */
+#endif
