@@ -20,16 +20,13 @@
  *
  */
 
-#ifndef GNUTLS_LIB_EXT_SRP_H
-#define GNUTLS_LIB_EXT_SRP_H
+#ifndef EXT_SRP_H
+#define EXT_SRP_H
 
-#include "hello_ext.h"
+#include <hello_ext.h>
 
-#define IS_SRP_KX(kx)                                         \
-	((kx == GNUTLS_KX_SRP || (kx == GNUTLS_KX_SRP_RSA) || \
-	  kx == GNUTLS_KX_SRP_DSS) ?                          \
-		 1 :                                          \
-		 0)
+#define IS_SRP_KX(kx) ((kx == GNUTLS_KX_SRP || (kx == GNUTLS_KX_SRP_RSA) || \
+	  kx == GNUTLS_KX_SRP_DSS)?1:0)
 
 #ifdef ENABLE_SRP
 
@@ -42,4 +39,4 @@ typedef struct {
 
 #endif
 
-#endif /* GNUTLS_LIB_EXT_SRP_H */
+#endif
