@@ -4,14 +4,14 @@
 
 #include <libtasn1.h>
 
-const ASN1_ARRAY_TYPE gnutls_asn1_tab[] = {
+const asn1_static_node gnutls_asn1_tab[] = {
   { "GNUTLS", 536872976, NULL },
   { NULL, 1073741836, NULL },
   { "RSAPublicKey", 1610612741, NULL },
   { "modulus", 1073741827, NULL },
   { "publicExponent", 3, NULL },
   { "RSAPrivateKey", 1610612741, NULL },
-  { "version", 1073741826, "Version"},
+  { "version", 1073741827, NULL },
   { "modulus", 1073741827, NULL },
   { "publicExponent", 1073741827, NULL },
   { "privateExponent", 1073741827, NULL },
@@ -21,9 +21,9 @@ const ASN1_ARRAY_TYPE gnutls_asn1_tab[] = {
   { "exponent2", 1073741827, NULL },
   { "coefficient", 1073741827, NULL },
   { "otherPrimeInfos", 16386, "OtherPrimeInfos"},
-  { "Version", 1610874883, NULL },
-  { "two-prime", 1073741825, "0"},
-  { "multi", 1, "1"},
+  { "ProvableSeed", 1610612741, NULL },
+  { "algorithm", 1073741836, NULL },
+  { "seed", 7, NULL },
   { "OtherPrimeInfos", 1612709899, NULL },
   { "MAX", 1074266122, "1"},
   { NULL, 2, "OtherPrimeInfo"},
@@ -37,9 +37,8 @@ const ASN1_ARRAY_TYPE gnutls_asn1_tab[] = {
   { "algorithm", 1, NULL },
   { "DigestInfo", 1610612741, NULL },
   { "digestAlgorithm", 1073741826, "DigestAlgorithmIdentifier"},
-  { "digest", 2, "Digest"},
+  { "digest", 7, NULL },
   { "DigestAlgorithmIdentifier", 1073741826, "AlgorithmIdentifier"},
-  { "Digest", 1073741831, NULL },
   { "DSAPublicKey", 1073741827, NULL },
   { "DSAParameters", 1610612741, NULL },
   { "p", 1073741827, NULL },
@@ -59,15 +58,42 @@ const ASN1_ARRAY_TYPE gnutls_asn1_tab[] = {
   { "prime", 1073741827, NULL },
   { "base", 1073741827, NULL },
   { "privateValueLength", 16387, NULL },
-  { "ECPoint", 1073741831, NULL },
   { "ECParameters", 1610612754, NULL },
   { "namedCurve", 12, NULL },
-  { "ECPrivateKey", 536870917, NULL },
+  { "ECPrivateKey", 1610612741, NULL },
   { "Version", 1073741827, NULL },
   { "privateKey", 1073741831, NULL },
   { "parameters", 1610637314, "ECParameters"},
   { NULL, 2056, "0"},
   { "publicKey", 536895494, NULL },
   { NULL, 2056, "1"},
+  { "PrincipalName", 1610612741, NULL },
+  { "name-type", 1610620931, NULL },
+  { NULL, 2056, "0"},
+  { "name-string", 536879115, NULL },
+  { NULL, 1073743880, "1"},
+  { NULL, 27, NULL },
+  { "KRB5PrincipalName", 1610612741, NULL },
+  { "realm", 1610620955, NULL },
+  { NULL, 2056, "0"},
+  { "principalName", 536879106, "PrincipalName"},
+  { NULL, 2056, "1"},
+  { "RSAPSSParameters", 1610612741, NULL },
+  { "hashAlgorithm", 1610637314, "AlgorithmIdentifier"},
+  { NULL, 2056, "0"},
+  { "maskGenAlgorithm", 1610637314, "AlgorithmIdentifier"},
+  { NULL, 2056, "1"},
+  { "saltLength", 1610653699, NULL },
+  { NULL, 1073741833, "20"},
+  { NULL, 2056, "2"},
+  { "trailerField", 536911875, NULL },
+  { NULL, 1073741833, "1"},
+  { NULL, 2056, "3"},
+  { "GOSTParameters", 1610612741, NULL },
+  { "publicKeyParamSet", 1073741836, NULL },
+  { "digestParamSet", 1073741836, NULL },
+  { "encryptionParamSet", 16396, NULL },
+  { "GOSTPrivateKey", 1073741831, NULL },
+  { "GOSTPrivateKeyOld", 3, NULL },
   { NULL, 0, NULL }
 };
