@@ -20,9 +20,6 @@
  *
  */
 
-#ifndef GNUTLS_LIB_KX_H
-#define GNUTLS_LIB_KX_H
-
 int _gnutls_send_server_kx_message(gnutls_session_t session, int again);
 int _gnutls_send_client_kx_message(gnutls_session_t session, int again);
 int _gnutls_recv_server_kx_message(gnutls_session_t session);
@@ -41,5 +38,3 @@ int _gnutls_recv_client_certificate_verify_message(gnutls_session_t
 void _gnutls_nss_keylog_write(gnutls_session_t session,
 			      const char *label,
 			      const uint8_t *secret, size_t secret_size);
-
-#endif /* GNUTLS_LIB_KX_H */
