@@ -15,7 +15,7 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  */
 
 #include "gnutls_int.h"
@@ -187,6 +187,8 @@ gnutls_pkcs11_copy_x509_crt2(const char *token_url,
 		a[1].value_len = cid->size;
 	}
 
+	/* we do not use the key usage flags; these are apparent from
+	 * the certificate itself. */
 	a[2].type = CKA_VALUE;
 	a[2].value = der;
 	a[2].value_len = der_size;
@@ -196,7 +198,6 @@ gnutls_pkcs11_copy_x509_crt2(const char *token_url,
 	a[4].type = CKA_CERTIFICATE_TYPE;
 	a[4].value = &type;
 	a[4].value_len = sizeof(type);
-	/* FIXME: copy key usage flags */
 
 	a_val = 5;
 
@@ -268,7 +269,6 @@ static void clean_pubkey(struct ck_attribute *a, unsigned a_val)
 			case CKA_EC_PARAMS:
 			case CKA_EC_POINT:
 				gnutls_free(a[i].value);
-				a[i].value = NULL;
 				break;
 		}
 	}
@@ -357,6 +357,29 @@ static int add_pubkey(gnutls_pubkey_t pubkey, struct ck_attribute *a, unsigned *
 		(*a_val)++;
 		break;
 	}
+	case GNUTLS_PK_EDDSA_ED25519: {
+		gnutls_datum_t params;
+
+		ret =
+		    _gnutls_x509_write_ecc_params(pubkey->params.curve,
+						  &params);
+		if (ret < 0) {
+			gnutls_assert();
+			return ret;
+		}
+
+		a[*a_val].type = CKA_EC_PARAMS;
+		a[*a_val].value = params.data;
+		a[*a_val].value_len = params.size;
+		(*a_val)++;
+
+		a[*a_val].type = CKA_EC_POINT;
+		a[*a_val].value = pubkey->params.raw_pub.data;
+		a[*a_val].value_len = pubkey->params.raw_pub.size;
+		(*a_val)++;
+		break;
+	}
+
 	default:
 		_gnutls_debug_log("requested writing public key of unsupported type %u\n", (unsigned)pk);
 		return gnutls_assert_val(GNUTLS_E_UNIMPLEMENTED_FEATURE);
@@ -693,7 +716,6 @@ gnutls_pkcs11_copy_x509_privkey2(const char *token_url,
 	pk = gnutls_x509_privkey_get_pk_algorithm(key);
 	FIX_KEY_USAGE(pk, key_usage);
 
-	/* FIXME: copy key usage flags */
 	a_val = 0;
 	a[a_val].type = CKA_CLASS;
 	a[a_val].value = &class;
@@ -730,7 +752,8 @@ gnutls_pkcs11_copy_x509_privkey2(const char *token_url,
 
 	if (pk == GNUTLS_PK_RSA) {
 		a[a_val].type = CKA_DECRYPT;
-		if (key_usage & (GNUTLS_KEY_ENCIPHER_ONLY|GNUTLS_KEY_DECIPHER_ONLY)) {
+		if ((key_usage & (GNUTLS_KEY_ENCIPHER_ONLY|GNUTLS_KEY_DECIPHER_ONLY)) ||
+		    (key_usage & GNUTLS_KEY_KEY_ENCIPHERMENT)) {
 			a[a_val].value = (void*)&tval;
 			a[a_val].value_len = sizeof(tval);
 		} else {
@@ -920,6 +943,32 @@ gnutls_pkcs11_copy_x509_privkey2(const char *token_url,
 
 			break;
 		}
+#ifdef HAVE_CKM_EDDSA
+	case GNUTLS_PK_EDDSA_ED25519:
+		{
+			ret =
+			    _gnutls_x509_write_ecc_params(key->params.curve,
+							  &p);
+			if (ret < 0) {
+				gnutls_assert();
+				goto cleanup;
+			}
+
+			type = CKK_EC_EDWARDS;
+
+			a[a_val].type = CKA_EC_PARAMS;
+			a[a_val].value = p.data;
+			a[a_val].value_len = p.size;
+			a_val++;
+
+			a[a_val].type = CKA_VALUE;
+			a[a_val].value = key->params.raw_priv.data;
+			a[a_val].value_len = key->params.raw_priv.size;
+			a_val++;
+
+			break;
+		}
+#endif
 	default:
 		gnutls_assert();
 		ret = GNUTLS_E_INVALID_REQUEST;
@@ -966,6 +1015,7 @@ gnutls_pkcs11_copy_x509_privkey2(const char *token_url,
 			break;
 		}
 	case GNUTLS_PK_EC:
+	case GNUTLS_PK_EDDSA_ED25519:
 		{
 			gnutls_free(p.data);
 			gnutls_free(x.data);
@@ -1193,6 +1243,8 @@ gnutls_pkcs11_token_init(const char *token_url,
 
 }
 
+#define L(x) ((x==NULL)?0:strlen(x))
+
 /**
  * gnutls_pkcs11_token_set_pin:
  * @token_url: A PKCS #11 URL specifying a token
@@ -1200,9 +1252,11 @@ gnutls_pkcs11_token_init(const char *token_url,
  * @newpin: new user's PIN
  * @flags: one of #gnutls_pin_flag_t.
  *
- * This function will modify or set a user's PIN for the given token. 
- * If it is called to set a user pin for first time the oldpin must
- * be NULL.
+ * This function will modify or set a user or administrator's PIN for
+ * the given token.  If it is called to set a PIN for first time
+ * the oldpin must be %NULL. When setting the admin's PIN with the
+ * %GNUTLS_PIN_SO flag, the @oldpin value must be provided (this requirement
+ * is relaxed after GnuTLS 3.6.5 since which the PIN will be requested if missing).
  *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
@@ -1240,7 +1294,8 @@ gnutls_pkcs11_token_set_pin(const char *token_url,
 		return ret;
 	}
 
-	if (oldpin == NULL) {
+	if (oldpin == NULL && !(flags & GNUTLS_PIN_SO)) {
+		/* This changes only the user PIN */
 		rv = pkcs11_init_pin(sinfo.module, sinfo.pks,
 				     (uint8_t *) newpin, strlen(newpin));
 		if (rv != CKR_OK) {
@@ -1251,9 +1306,32 @@ gnutls_pkcs11_token_set_pin(const char *token_url,
 			goto finish;
 		}
 	} else {
+		struct p11_kit_pin *pin;
+		unsigned oldpin_size;
+
+		oldpin_size = L(oldpin);
+
+		if (!(sinfo.tinfo.flags & CKF_PROTECTED_AUTHENTICATION_PATH)) {
+			if (newpin == NULL)
+				return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+			if (oldpin == NULL) {
+				struct pin_info_st pin_info;
+				memset(&pin_info, 0, sizeof(pin_info));
+
+				ret = pkcs11_retrieve_pin(&pin_info, info, &sinfo.tinfo, 0, CKU_SO, &pin);
+				if (ret < 0) {
+					gnutls_assert();
+					goto finish;
+				}
+				oldpin = (const char*)p11_kit_pin_get_value(pin, NULL);
+				oldpin_size = p11_kit_pin_get_length(pin);
+			}
+		}
+
 		rv = pkcs11_set_pin(sinfo.module, sinfo.pks,
-				    oldpin, strlen(oldpin),
-				    newpin, strlen(newpin));
+				    oldpin, oldpin_size,
+				    newpin, L(newpin));
 		if (rv != CKR_OK) {
 			gnutls_assert();
 			_gnutls_debug_log("p11: %s\n",
