@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2002-2015 Free Software Foundation, Inc.
  * Copyright (C) 2014-2015 Nikos Mavrogiannopoulos
+ * Copyright (C) 2016-2017 Red Hat, Inc.
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -17,7 +18,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  *
  */
 
@@ -26,247 +27,12 @@
 
 #include "gnutls_int.h"
 #include "errors.h"
+#include "secrets.h"
 #include <num.h>
 #include <state.h>
 #include <algorithms.h>
 
-#define MAX_PRF_BYTES 200
-#define MAX_SEED_SIZE 200
-
-inline static int
-_gnutls_cal_PRF_A(const mac_entry_st * me,
-		  const void *secret, int secret_size,
-		  const void *seed, int seed_size, void *result)
-{
-	int ret;
-
-	ret =
-	    _gnutls_mac_fast(me->id, secret, secret_size, seed, seed_size,
-			     result);
-	if (ret < 0)
-		return gnutls_assert_val(ret);
-
-	return 0;
-}
-
-/* Produces "total_bytes" bytes using the hash algorithm specified.
- * (used in the PRF function)
- */
-static int
-P_hash(gnutls_mac_algorithm_t algorithm,
-       const uint8_t * secret, int secret_size,
-       const uint8_t * seed, int seed_size, int total_bytes, uint8_t * ret)
-{
-
-	mac_hd_st td2;
-	int i, times, how, blocksize, A_size;
-	uint8_t final[MAX_HASH_SIZE], Atmp[MAX_SEED_SIZE];
-	int output_bytes, result;
-	const mac_entry_st *me = mac_to_entry(algorithm);
-
-	blocksize = _gnutls_mac_get_algo_len(me);
-
-	if (seed_size > MAX_SEED_SIZE || total_bytes <= 0 || blocksize == 0) {
-		gnutls_assert();
-		return GNUTLS_E_INTERNAL_ERROR;
-	}
-
-	output_bytes = 0;
-	do {
-		output_bytes += blocksize;
-	}
-	while (output_bytes < total_bytes);
-
-	/* calculate A(0) */
-
-	memcpy(Atmp, seed, seed_size);
-	A_size = seed_size;
-
-	times = output_bytes / blocksize;
-
-	for (i = 0; i < times; i++) {
-		result = _gnutls_mac_init(&td2, me, secret, secret_size);
-		if (result < 0) {
-			gnutls_assert();
-			return result;
-		}
-
-		/* here we calculate A(i+1) */
-		if ((result =
-		     _gnutls_cal_PRF_A(me, secret, secret_size, Atmp,
-				       A_size, Atmp)) < 0) {
-			gnutls_assert();
-			_gnutls_mac_deinit(&td2, final);
-			return result;
-		}
-
-		A_size = blocksize;
-
-		_gnutls_mac(&td2, Atmp, A_size);
-		_gnutls_mac(&td2, seed, seed_size);
-		_gnutls_mac_deinit(&td2, final);
-
-		if ((1 + i) * blocksize < total_bytes) {
-			how = blocksize;
-		} else {
-			how = total_bytes - (i) * blocksize;
-		}
-
-		if (how > 0) {
-			memcpy(&ret[i * blocksize], final, how);
-		}
-	}
-
-	return 0;
-}
-
-/* This function operates as _gnutls_PRF(), but does not require
- * a pointer to the current session. It takes the @mac algorithm
- * explicitly. For legacy TLS/SSL sessions before TLS 1.2 the MAC
- * must be set to %GNUTLS_MAC_UNKNOWN.
- */
-static int
-_gnutls_PRF_raw(gnutls_mac_algorithm_t mac,
-		const uint8_t * secret, unsigned int secret_size,
-		const char *label, int label_size, const uint8_t * seed,
-		int seed_size, int total_bytes, void *ret)
-{
-	int l_s, s_seed_size;
-	const uint8_t *s1, *s2;
-	uint8_t s_seed[MAX_SEED_SIZE];
-	uint8_t o1[MAX_PRF_BYTES], o2[MAX_PRF_BYTES];
-	int result;
-
-	if (total_bytes > MAX_PRF_BYTES) {
-		gnutls_assert();
-		return GNUTLS_E_INTERNAL_ERROR;
-	}
-	/* label+seed = s_seed */
-	s_seed_size = seed_size + label_size;
-
-	if (s_seed_size > MAX_SEED_SIZE) {
-		gnutls_assert();
-		return GNUTLS_E_INTERNAL_ERROR;
-	}
-
-	memcpy(s_seed, label, label_size);
-	memcpy(&s_seed[label_size], seed, seed_size);
-
-	if (mac != GNUTLS_MAC_UNKNOWN) {
-		result =
-		    P_hash(mac, secret, secret_size,
-			   s_seed, s_seed_size,
-			   total_bytes, ret);
-		if (result < 0) {
-			gnutls_assert();
-			return result;
-		}
-	} else {
-		l_s = secret_size / 2;
-
-		s1 = &secret[0];
-		s2 = &secret[l_s];
-
-		if (secret_size % 2 != 0) {
-			l_s++;
-		}
-
-		result =
-		    P_hash(GNUTLS_MAC_MD5, s1, l_s, s_seed, s_seed_size,
-			   total_bytes, o1);
-		if (result < 0) {
-			gnutls_assert();
-			return result;
-		}
-
-		result =
-		    P_hash(GNUTLS_MAC_SHA1, s2, l_s, s_seed, s_seed_size,
-			   total_bytes, o2);
-		if (result < 0) {
-			gnutls_assert();
-			return result;
-		}
-
-		memxor(o1, o2, total_bytes);
-
-		memcpy(ret, o1, total_bytes);
-	}
-
-	return 0;		/* ok */
-}
-
-
-/* The PRF function expands a given secret 
- * needed by the TLS specification. ret must have a least total_bytes
- * available.
- */
-int
-_gnutls_PRF(gnutls_session_t session,
-	    const uint8_t * secret, unsigned int secret_size,
-	    const char *label, int label_size, const uint8_t * seed,
-	    int seed_size, int total_bytes, void *ret)
-{
-	const version_entry_st *ver = get_version(session);
-
-	if (_gnutls_version_has_selectable_prf(ver)) {
-		return _gnutls_PRF_raw(
-			_gnutls_cipher_suite_get_prf(session->security_parameters.cipher_suite),
-			secret, secret_size,
-			label, label_size,
-			seed, seed_size,
-			total_bytes,
-			ret);
-	} else {
-		return _gnutls_PRF_raw(
-			GNUTLS_MAC_UNKNOWN,
-			secret, secret_size,
-			label, label_size,
-			seed, seed_size,
-			total_bytes,
-			ret);
-	}
-}
-
-#ifdef ENABLE_FIPS140
-int
-_gnutls_prf_raw(gnutls_mac_algorithm_t mac,
-		size_t master_size, const void *master,
-		size_t label_size, const char *label,
-		size_t seed_size, const char *seed, size_t outsize,
-		char *out);
-
-/*-
- * _gnutls_prf_raw:
- * @mac: the MAC algorithm to use, set to %GNUTLS_MAC_UNKNOWN for the TLS1.0 mac
- * @master_size: length of the @master variable.
- * @master: the master secret used in PRF computation
- * @label_size: length of the @label variable.
- * @label: label used in PRF computation, typically a short string.
- * @seed_size: length of the @seed variable.
- * @seed: optional extra data to seed the PRF with.
- * @outsize: size of pre-allocated output buffer to hold the output.
- * @out: pre-allocated buffer to hold the generated data.
- *
- * Apply the TLS Pseudo-Random-Function (PRF) on the master secret
- * and the provided data.
- *
- * Returns: %GNUTLS_E_SUCCESS on success, or an error code.
- -*/
-int
-_gnutls_prf_raw(gnutls_mac_algorithm_t mac,
-		size_t master_size, const void *master,
-		size_t label_size, const char *label,
-		size_t seed_size, const char *seed, size_t outsize,
-		char *out)
-{
-	return _gnutls_PRF_raw(mac,
-			  master, master_size,
-			  label, label_size,
-			  (uint8_t *) seed, seed_size,
-			  outsize, out);
-
-}
-#endif
+#define EXPORTER_LABEL "exporter"
 
 /**
  * gnutls_prf_raw:
@@ -295,6 +61,11 @@ _gnutls_prf_raw(gnutls_mac_algorithm_t mac,
  * client and server random fields directly, and is recommended if you
  * want to generate pseudo random data unique for each session.
  *
+ * Note: This function will only operate under TLS versions prior to 1.3.
+ * In TLS1.3 the use of PRF is replaced with HKDF and the generic
+ * exporters like gnutls_prf_rfc5705() should be used instead. Under
+ * TLS1.3 this function returns %GNUTLS_E_INVALID_REQUEST.
+ *
  * Returns: %GNUTLS_E_SUCCESS on success, or an error code.
  **/
 int
@@ -305,13 +76,16 @@ gnutls_prf_raw(gnutls_session_t session,
 	       char *out)
 {
 	int ret;
+	const version_entry_st *vers = get_version(session);
 
-	ret = _gnutls_PRF(session,
-			  session->security_parameters.master_secret,
-			  GNUTLS_MASTER_SIZE,
-			  label,
-			  label_size, (uint8_t *) seed, seed_size, outsize,
-			  out);
+	if (vers && vers->tls13_sem)
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+	ret = _gnutls_prf_raw(session->security_parameters.prf->id,
+			  GNUTLS_MASTER_SIZE, session->security_parameters.master_secret,
+			  label_size, label,
+			  seed_size, (uint8_t *) seed,
+			  outsize, out);
 
 	return ret;
 }
@@ -326,15 +100,18 @@ gnutls_prf_raw(gnutls_session_t session,
  * @outsize: size of pre-allocated output buffer to hold the output.
  * @out: pre-allocated buffer to hold the generated data.
  *
- * Applies the TLS Pseudo-Random-Function (PRF) on the master secret
- * and the provided data, seeded with the client and server random fields,
+ * Exports keyring material from TLS/DTLS session to an application,
  * as specified in RFC5705.
  *
+ * In the TLS versions prior to 1.3, it applies the TLS
+ * Pseudo-Random-Function (PRF) on the master secret and the provided
+ * data, seeded with the client and server random fields.
+ *
+ * In TLS 1.3, it applies HKDF on the exporter master secret derived
+ * from the master secret.
+ *
  * The @label variable usually contains a string denoting the purpose
- * for the generated data.  The @server_random_first indicates whether
- * the client random field or the server random field should be first
- * in the seed.  Non-zero indicates that the server random field is first,
- * 0 that the client random field is first.
+ * for the generated data.
  *
  * The @context variable can be used to add more data to the seed, after
  * the random variables.  It can be used to make sure the
@@ -356,30 +133,60 @@ gnutls_prf_rfc5705(gnutls_session_t session,
 		   size_t context_size, const char *context,
 		   size_t outsize, char *out)
 {
-	char *pctx = NULL;
+	const version_entry_st *vers = get_version(session);
 	int ret;
 
-	if (context != NULL && context_size > 65535)  {
-		gnutls_assert();
-		return GNUTLS_E_INVALID_REQUEST;
-	}
+	if (vers && vers->tls13_sem) {
+		uint8_t secret[MAX_HASH_SIZE];
+		uint8_t digest[MAX_HASH_SIZE];
+		unsigned digest_size = session->security_parameters.prf->output_size;
 
-	if (context != NULL) {
-		pctx = gnutls_malloc(context_size+2);
-		if (!pctx) {
+		/* exporter_master_secret might not be set, when
+		 * handshake is in progress */
+		if (session->internals.handshake_in_progress) {
 			gnutls_assert();
-			return GNUTLS_E_MEMORY_ERROR;
+			return GNUTLS_E_INVALID_REQUEST;
 		}
 
-		memcpy(pctx+2, context, context_size);
-		_gnutls_write_uint16(context_size, (void*)pctx);
-		context_size += 2;
+		ret = _tls13_derive_secret(session, label, label_size, NULL, 0,
+					   session->key.proto.tls13.ap_expkey, secret);
+		if (ret < 0)
+			return gnutls_assert_val(ret);
+
+		ret = gnutls_hash_fast((gnutls_digest_algorithm_t)session->security_parameters.prf->id,
+				       context, context_size, digest);
+		if (ret < 0)
+			return gnutls_assert_val(ret);
+
+		ret = _tls13_expand_secret(session, EXPORTER_LABEL, sizeof(EXPORTER_LABEL)-1,
+					   digest, digest_size,
+					   secret, outsize, out);
+	} else {
+		char *pctx = NULL;
+
+		if (context != NULL && context_size > 65535)  {
+			gnutls_assert();
+			return GNUTLS_E_INVALID_REQUEST;
+		}
+
+		if (context != NULL) {
+			pctx = gnutls_malloc(context_size+2);
+			if (!pctx) {
+				gnutls_assert();
+				return GNUTLS_E_MEMORY_ERROR;
+			}
+
+			memcpy(pctx+2, context, context_size);
+			_gnutls_write_uint16(context_size, (void*)pctx);
+			context_size += 2;
+		}
+
+		ret = gnutls_prf(session, label_size, label, 0,
+				 context_size, pctx, outsize, out);
+
+		gnutls_free(pctx);
 	}
 
-	ret = gnutls_prf(session, label_size, label, 0,
-			 context_size, pctx, outsize, out);
-
-	gnutls_free(pctx);
 	return ret;
 }
 
@@ -412,7 +219,9 @@ gnutls_prf_rfc5705(gnutls_session_t session,
  * The output is placed in @out, which must be pre-allocated.
  *
  * Note: This function produces identical output with gnutls_prf_rfc5705()
- * when @server_random_first is set to 0 and @extra is %NULL.
+ * when @server_random_first is set to 0 and @extra is %NULL. Under TLS1.3
+ * this function will only operate when these conditions are true, or otherwise
+ * return %GNUTLS_E_INVALID_REQUEST.
  *
  * Returns: %GNUTLS_E_SUCCESS on success, or an error code.
  **/
@@ -426,8 +235,17 @@ gnutls_prf(gnutls_session_t session,
 {
 	int ret;
 	uint8_t *seed;
+	const version_entry_st *vers = get_version(session);
 	size_t seedsize = 2 * GNUTLS_RANDOM_SIZE + extra_size;
 
+	if (vers && vers->tls13_sem) {
+		if (extra == NULL && server_random_first == 0)
+			return gnutls_prf_rfc5705(session, label_size, label,
+						  extra_size, extra, outsize, out);
+		else
+			return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+	}
+
 	seed = gnutls_malloc(seedsize);
 	if (!seed) {
 		gnutls_assert();
@@ -448,10 +266,11 @@ gnutls_prf(gnutls_session_t session,
 	}
 
 	ret =
-	    _gnutls_PRF(session,
-			session->security_parameters.master_secret,
-			GNUTLS_MASTER_SIZE, label, label_size, seed,
-			seedsize, outsize, out);
+	    _gnutls_prf_raw(session->security_parameters.prf->id,
+			GNUTLS_MASTER_SIZE, session->security_parameters.master_secret,
+			label_size, label,
+			seedsize, seed,
+			outsize, out);
 
 	gnutls_free(seed);
 
