@@ -20,15 +20,11 @@
  *
  */
 
-#ifndef GNUTLS_LIB_SUPPLEMENTAL_H
-#define GNUTLS_LIB_SUPPLEMENTAL_H
-
 #include "gnutls_int.h"
 
-int _gnutls_parse_supplemental(gnutls_session_t session, const uint8_t *data,
-			       int data_size);
-int _gnutls_gen_supplemental(gnutls_session_t session, gnutls_buffer_st *buf);
+int _gnutls_parse_supplemental(gnutls_session_t session,
+			       const uint8_t * data, int data_size);
+int _gnutls_gen_supplemental(gnutls_session_t session,
+			     gnutls_buffer_st * buf);
 
 void _gnutls_supplemental_deinit(void);
-
-#endif /* GNUTLS_LIB_SUPPLEMENTAL_H */
