@@ -20,22 +20,24 @@
  *
  */
 
-#ifndef GNUTLS_LIB_TLS13_SIG_H
-#define GNUTLS_LIB_TLS13_SIG_H
+#ifndef _TLS13_SIG_H
+# define _TLS13_SIG_H
 
 #include "gnutls_int.h"
 
-int _gnutls13_handshake_verify_data(gnutls_session_t session,
-				    unsigned verify_flags,
-				    gnutls_pcert_st *cert,
-				    const gnutls_datum_t *context,
-				    const gnutls_datum_t *signature,
-				    const gnutls_sign_entry_st *se);
+int
+_gnutls13_handshake_verify_data(gnutls_session_t session,
+			        unsigned verify_flags,
+			        gnutls_pcert_st *cert,
+			        const gnutls_datum_t *context,
+			        const gnutls_datum_t *signature,
+			        const gnutls_sign_entry_st *se);
 
-int _gnutls13_handshake_sign_data(gnutls_session_t session,
-				  gnutls_pcert_st *cert, gnutls_privkey_t pkey,
-				  const gnutls_datum_t *context,
-				  gnutls_datum_t *signature,
-				  const gnutls_sign_entry_st *se);
+int
+_gnutls13_handshake_sign_data(gnutls_session_t session,
+			      gnutls_pcert_st * cert, gnutls_privkey_t pkey,
+			      const gnutls_datum_t *context,
+			      gnutls_datum_t * signature,
+			      const gnutls_sign_entry_st *se);
 
-#endif /* GNUTLS_LIB_TLS13_SIG_H */
+#endif
