@@ -1,5 +1,5 @@
 /* Normalization of UTF-16 strings.
-   Copyright (C) 2009-2021 Free Software Foundation, Inc.
+   Copyright (C) 2009-2019 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2009.
 
    This program is free software: you can redistribute it and/or
@@ -37,7 +37,7 @@
 #include "unistr.h"
 #include "unictype.h"
 #include "normalize-internal.h"
-#include "uninorm/decompose-internal.h"
+#include "decompose-internal.h"
 
 #define FUNC u16_normalize
 #define UNIT uint16_t
