@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2003-2016 Free Software Foundation, Inc.
- * Copyright (C) 2015-2016 Red Hat, Inc.
+ * Copyright (C) 2015-2017 Red Hat, Inc.
  *
  * This file is part of GnuTLS.
  *
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -63,7 +63,10 @@ static unsigned long file_size(FILE *fp)
 		return 0;
 
 	size = ftell(fp);
-	fseek(fp, cur, SEEK_SET);
+	if (fseek(fp, cur, SEEK_SET) == -1) {
+		fprintf(stderr, "Error reading file size\n");
+		app_exit(1);
+	}
 	return size;
 }
 
@@ -82,7 +85,7 @@ void fix_lbuffer(unsigned long size)
 
 	if (lbuffer == NULL) {
 		fprintf(stderr, "memory error");
-		exit(1);
+		app_exit(1);
 	}
 }
 
@@ -118,7 +121,7 @@ gnutls_datum_t *load_secret_key(int mand, common_info_st * info)
 	if (info->secret_key == NULL) {
 		if (mand) {
 			fprintf(stderr, "missing --secret-key\n");
-			exit(1);
+			app_exit(1);
 		} else
 			return NULL;
 	}
@@ -129,7 +132,7 @@ gnutls_datum_t *load_secret_key(int mand, common_info_st * info)
 	ret = gnutls_hex_decode(&hex_key, raw_key, &raw_key_size);
 	if (ret < 0) {
 		fprintf(stderr, "hex_decode: %s\n", gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 
 	key.data = (void *) raw_key;
@@ -173,7 +176,7 @@ static gnutls_privkey_t _load_privkey(gnutls_datum_t * dat,
 	ret = gnutls_privkey_init(&key);
 	if (ret < 0) {
 		fprintf(stderr, "privkey_init: %s\n", gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 
 	ret =
@@ -191,13 +194,13 @@ static gnutls_privkey_t _load_privkey(gnutls_datum_t * dat,
 		fprintf(stderr,
 			"import error: could not find a valid PEM header; "
 			"check if your key is PKCS #12 encoded\n");
-		exit(1);
+		app_exit(1);
 	}
 
 	if (ret < 0) {
-		fprintf(stderr, "importing --load-privkey: %s: %s\n",
+		fprintf(stderr, "error loading file at --load-privkey: %s: %s\n",
 			info->privkey, gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 
 	return key;
@@ -211,14 +214,14 @@ static gnutls_privkey_t _load_url_privkey(const char *url)
 	ret = gnutls_privkey_init(&key);
 	if (ret < 0) {
 		fprintf(stderr, "privkey_init: %s\n", gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 
 	ret = gnutls_privkey_import_url(key, url, 0);
 	if (ret < 0) {
-		fprintf(stderr, "importing key: %s: %s\n",
+		fprintf(stderr, "error importing key at %s: %s\n",
 			url, gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 
 	return key;
@@ -234,14 +237,14 @@ static gnutls_pubkey_t _load_url_pubkey(const char *url)
 	if (ret < 0) {
 		fprintf(stderr, "Error in %s:%d: %s\n", __func__, __LINE__,
 			gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 
 	ret = gnutls_pubkey_import_url(pubkey, url, obj_flags);
 	if (ret < 0) {
 		fprintf(stderr, "Error in %s:%d: %s: %s\n", __func__,
 			__LINE__, gnutls_strerror(ret), url);
-		exit(1);
+		app_exit(1);
 	}
 
 	return pubkey;
@@ -261,7 +264,7 @@ gnutls_privkey_t load_private_key(int mand, common_info_st * info)
 
 	if (info->privkey == NULL) {
 		fprintf(stderr, "missing --load-privkey\n");
-		exit(1);
+		app_exit(1);
 	}
 
 	if (gnutls_url_is_supported(info->privkey) != 0)
@@ -271,9 +274,9 @@ gnutls_privkey_t load_private_key(int mand, common_info_st * info)
 	dat.size = size;
 
 	if (!dat.data) {
-		fprintf(stderr, "error reading --load-privkey: %s\n",
+		fprintf(stderr, "error reading file at --load-privkey: %s\n",
 			info->privkey);
-		exit(1);
+		app_exit(1);
 	}
 
 	key = _load_privkey(&dat, info);
@@ -301,22 +304,22 @@ load_x509_private_key(int mand, common_info_st * info)
 
 	if (info->privkey == NULL) {
 		fprintf(stderr, "missing --load-privkey\n");
-		exit(1);
+		app_exit(1);
 	}
 
 	ret = gnutls_x509_privkey_init(&key);
 	if (ret < 0) {
 		fprintf(stderr, "privkey_init: %s\n", gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 
 	dat.data = (void *) read_binary_file(info->privkey, &size);
 	dat.size = size;
 
 	if (!dat.data) {
-		fprintf(stderr, "error reading --load-privkey: %s\n",
+		fprintf(stderr, "error reading file at --load-privkey: %s\n",
 			info->privkey);
-		exit(1);
+		app_exit(1);
 	}
 
 	if (info->pkcs8) {
@@ -346,13 +349,13 @@ load_x509_private_key(int mand, common_info_st * info)
 		fprintf(stderr,
 			"import error: could not find a valid PEM header; "
 			"check if your key is PEM encoded\n");
-		exit(1);
+		app_exit(1);
 	}
 
 	if (ret < 0) {
-		fprintf(stderr, "importing --load-privkey: %s: %s\n",
+		fprintf(stderr, "error importing private key: %s: %s\n",
 			info->privkey, gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 
 	return key;
@@ -392,6 +395,7 @@ gnutls_x509_crt_t *load_cert_list(int mand, size_t * crt_size,
 	gnutls_datum_t dat;
 	unsigned size;
 	unsigned int crt_max;
+	unsigned flags = 0;
 
 	*crt_size = 0;
 	if (info->verbose)
@@ -400,7 +404,7 @@ gnutls_x509_crt_t *load_cert_list(int mand, size_t * crt_size,
 	if (info->cert == NULL) {
 		if (mand) {
 			fprintf(stderr, "missing --load-certificate\n");
-			exit(1);
+			app_exit(1);
 		} else
 			return NULL;
 	}
@@ -408,7 +412,7 @@ gnutls_x509_crt_t *load_cert_list(int mand, size_t * crt_size,
 	fd = fopen(info->cert, "r");
 	if (fd == NULL) {
 		fprintf(stderr, "Could not open %s\n", info->cert);
-		exit(1);
+		app_exit(1);
 	}
 
 	fix_lbuffer(file_size(fd));
@@ -421,10 +425,13 @@ gnutls_x509_crt_t *load_cert_list(int mand, size_t * crt_size,
 	dat.data = (void *) lbuffer;
 	dat.size = size;
 
-	ret = gnutls_x509_crt_list_import2(&crt, &crt_max, &dat, GNUTLS_X509_FMT_PEM, 0);
+	if (info->sort_chain)
+		flags |= GNUTLS_X509_CRT_LIST_SORT;
+
+	ret = gnutls_x509_crt_list_import2(&crt, &crt_max, &dat, GNUTLS_X509_FMT_PEM, flags);
 	if (ret < 0) {
 		fprintf(stderr, "Error loading certificates: %s\n", gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 
 	*crt_size = crt_max;
@@ -455,7 +462,7 @@ gnutls_x509_crl_t *load_crl_list(int mand, size_t * crl_size,
 	if (info->crl == NULL) {
 		if (mand) {
 			fprintf(stderr, "missing --load-crl\n");
-			exit(1);
+			app_exit(1);
 		} else
 			return NULL;
 	}
@@ -463,7 +470,7 @@ gnutls_x509_crl_t *load_crl_list(int mand, size_t * crl_size,
 	fd = fopen(info->crl, "r");
 	if (fd == NULL) {
 		fprintf(stderr, "Could not open %s\n", info->crl);
-		exit(1);
+		app_exit(1);
 	}
 
 	fix_lbuffer(file_size(fd));
@@ -477,9 +484,12 @@ gnutls_x509_crl_t *load_crl_list(int mand, size_t * crl_size,
 	dat.size = size;
 
 	ret = gnutls_x509_crl_list_import2(&crl, &crl_max, &dat, GNUTLS_X509_FMT_PEM, 0);
+	if (ret == GNUTLS_E_BASE64_DECODING_ERROR) {
+		ret = gnutls_x509_crl_list_import2(&crl, &crl_max, &dat, GNUTLS_X509_FMT_DER, 0);
+	}
 	if (ret < 0) {
 		fprintf(stderr, "Error loading CRLs: %s\n", gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 
 	*crl_size = crl_max;
@@ -506,30 +516,30 @@ gnutls_x509_crq_t load_request(common_info_st * info)
 	ret = gnutls_x509_crq_init(&crq);
 	if (ret < 0) {
 		fprintf(stderr, "crq_init: %s\n", gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 
 	dat.data = (void *) read_binary_file(info->request, &size);
 	dat.size = size;
 
 	if (!dat.data) {
-		fprintf(stderr, "error reading --load-request: %s\n",
+		fprintf(stderr, "error reading file at --load-request: %s\n",
 			info->request);
-		exit(1);
+		app_exit(1);
 	}
 
 	ret = gnutls_x509_crq_import(crq, &dat, info->incert_format);
 	if (ret == GNUTLS_E_BASE64_UNEXPECTED_HEADER_ERROR) {
 		fprintf(stderr,
 			"import error: could not find a valid PEM header\n");
-		exit(1);
+		app_exit(1);
 	}
 
 	free(dat.data);
 	if (ret < 0) {
-		fprintf(stderr, "importing --load-request: %s: %s\n",
+		fprintf(stderr, "error importing certificate request: %s: %s\n",
 			info->request, gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 	return crq;
 }
@@ -544,7 +554,7 @@ gnutls_privkey_t load_ca_private_key(common_info_st * info)
 
 	if (info->ca_privkey == NULL) {
 		fprintf(stderr, "missing --load-ca-privkey\n");
-		exit(1);
+		app_exit(1);
 	}
 
 	if (gnutls_url_is_supported(info->ca_privkey) != 0)
@@ -554,9 +564,9 @@ gnutls_privkey_t load_ca_private_key(common_info_st * info)
 	dat.size = size;
 
 	if (!dat.data) {
-		fprintf(stderr, "error reading --load-ca-privkey: %s\n",
+		fprintf(stderr, "error reading file at --load-ca-privkey: %s\n",
 			info->ca_privkey);
-		exit(1);
+		app_exit(1);
 	}
 
 	key = _load_privkey(&dat, info);
@@ -581,30 +591,40 @@ gnutls_x509_crt_t load_ca_cert(unsigned mand, common_info_st * info)
 
 	if (info->ca == NULL) {
 		fprintf(stderr, "missing --load-ca-certificate\n");
-		exit(1);
+		app_exit(1);
 	}
 
 	ret = gnutls_x509_crt_init(&crt);
 	if (ret < 0) {
 		fprintf(stderr, "crt_init: %s\n", gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
+	}
+
+	if (gnutls_url_is_supported(info->ca) != 0) {
+		ret = gnutls_x509_crt_import_url(crt, info->ca, 0);
+		if (ret < 0) {
+			fprintf(stderr, "error importing CA certificate: %s: %s\n",
+				info->ca, gnutls_strerror(ret));
+			app_exit(1);
+		}
+		return crt;
 	}
 
 	dat.data = (void *) read_binary_file(info->ca, &size);
 	dat.size = size;
 
 	if (!dat.data) {
-		fprintf(stderr, "error reading --load-ca-certificate: %s\n",
+		fprintf(stderr, "error reading file at --load-ca-certificate: %s\n",
 			info->ca);
-		exit(1);
+		app_exit(1);
 	}
 
 	ret = gnutls_x509_crt_import(crt, &dat, info->incert_format);
 	free(dat.data);
 	if (ret < 0) {
-		fprintf(stderr, "importing --load-ca-certificate: %s: %s\n",
+		fprintf(stderr, "error importing CA certificate: %s: %s\n",
 			info->ca, gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 
 	return crt;
@@ -625,7 +645,7 @@ gnutls_pubkey_t load_pubkey(int mand, common_info_st * info)
 
 	if (info->pubkey == NULL) {
 		fprintf(stderr, "missing --load-pubkey\n");
-		exit(1);
+		app_exit(1);
 	}
 
 	if (gnutls_url_is_supported(info->pubkey) != 0)
@@ -634,15 +654,15 @@ gnutls_pubkey_t load_pubkey(int mand, common_info_st * info)
 	ret = gnutls_pubkey_init(&key);
 	if (ret < 0) {
 		fprintf(stderr, "privkey_init: %s\n", gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 
 	dat.data = (void *) read_binary_file(info->pubkey, &size);
 	dat.size = size;
 
 	if (!dat.data) {
-		fprintf(stderr, "error reading --load-pubkey: %s\n", info->pubkey);
-		exit(1);
+		fprintf(stderr, "error reading file at --load-pubkey: %s\n", info->pubkey);
+		app_exit(1);
 	}
 
 	ret = gnutls_pubkey_import(key, &dat, info->incert_format);
@@ -652,12 +672,12 @@ gnutls_pubkey_t load_pubkey(int mand, common_info_st * info)
 			fprintf(stderr,
 				"import error: could not find a valid PEM header; "
 				"check if your key has the PUBLIC KEY header\n");
-			exit(1);
+			app_exit(1);
 		}
 	} else if (ret < 0) {
-		fprintf(stderr, "importing --load-pubkey: %s: %s\n",
+		fprintf(stderr, "importing public key: %s: %s\n",
 			info->pubkey, gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 
 	free(dat.data);
@@ -675,7 +695,7 @@ gnutls_pubkey_t load_public_key_or_import(int mand,
 	if (ret < 0) {
 		fprintf(stderr, "gnutls_pubkey_init: %s\n",
 			gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 
 	if (!privkey || (ret = gnutls_pubkey_import_privkey(pubkey, privkey, 0, 0)) < 0) {	/* could not get (e.g. on PKCS #11 */
@@ -683,7 +703,7 @@ gnutls_pubkey_t load_public_key_or_import(int mand,
 		pubkey = load_pubkey(0, info);
 		if (pubkey == NULL && mand) {
 			fprintf(stderr, "Could not determine the public key for the operation.\nYou must specify --load-privkey or --load-pubkey if missing.\n");
-			exit(1);
+			app_exit(1);
 		}
 	}
 
@@ -747,7 +767,7 @@ gnutls_sec_param_t str_to_sec_param(const char *str)
 	} else {
 		fprintf(stderr, "Unknown security parameter string: %s\n",
 			str);
-		exit(1);
+		app_exit(1);
 	}
 
 }
@@ -789,14 +809,20 @@ static void print_head(FILE * out, const char *txt, unsigned int size,
 {
 	unsigned i;
 	char *p, *ntxt;
+	int ret;
 
 	if (cprint != 0) {
 		if (size > 0)
-			asprintf(&ntxt, "const unsigned char %s[%u] =",
-				 txt, size);
+			ret = asprintf(&ntxt, "const unsigned char %s[%u] =",
+				       txt, size);
 		else
-			asprintf(&ntxt, "const unsigned char %s[] =\n",
-				 txt);
+			ret = asprintf(&ntxt, "const unsigned char %s[] =\n",
+				       txt);
+
+		if (ret == -1) {
+			fprintf(stderr, "memory error\n");
+			app_exit(1);
+		}
 
 		p = strstr(ntxt, "char");
 		p += 5;
@@ -856,7 +882,7 @@ const gnutls_ecc_curve_t *list, *p;
 		       gnutls_ecc_curve_get_name(*p));
 		p++;
 	}
-	exit(1);
+	app_exit(1);
 }
 
 void
@@ -871,17 +897,74 @@ print_ecc_pkey(FILE * outfile, gnutls_ecc_curve_t curve,
 		fprintf(outfile, "curve:\t%s\n",
 			gnutls_ecc_curve_get_name(curve));
 
-	if (k) {
+	if (k && k->data) {
 		print_head(outfile, "private key", k->size, cprint);
 		print_hex_datum(outfile, k, cprint);
 	}
+
+	if (x && x->data) {
+		print_head(outfile, "x", x->size, cprint);
+		print_hex_datum(outfile, x, cprint);
+	}
+
+	if (y && y->data) {
+		print_head(outfile, "y", y->size, cprint);
+		print_hex_datum(outfile, y, cprint);
+	}
+}
+
+static void reverse_datum(gnutls_datum_t *d)
+{
+	unsigned int i;
+	unsigned char c;
+
+	for (i = 0; i < d->size / 2; i++) {
+		c = d->data[i];
+		d->data[i] = d->data[d->size - i - 1];
+		d->data[d->size - i - 1] = c;
+	}
+}
+
+void
+print_gost_pkey(FILE * outfile, gnutls_ecc_curve_t curve,
+	       gnutls_digest_algorithm_t digest, gnutls_gost_paramset_t paramset,
+	       gnutls_datum_t * k, gnutls_datum_t * x, gnutls_datum_t * y,
+	       int cprint)
+{
+	if (cprint != 0)
+		fprintf(outfile, "/* curve: %s */\n",
+			gnutls_ecc_curve_get_name(curve));
+	else
+		fprintf(outfile, "curve:\t%s\n",
+			gnutls_ecc_curve_get_name(curve));
+
+	if (cprint != 0)
+		fprintf(outfile, "/* digest: %s */\n",
+			gnutls_digest_get_name(digest));
+	else
+		fprintf(outfile, "digest:\t%s\n",
+			gnutls_digest_get_name(digest));
+
+	if (cprint != 0)
+		fprintf(outfile, "/* paramset: %s */\n",
+			gnutls_gost_paramset_get_name(paramset));
+	else
+		fprintf(outfile, "paramset:\t%s\n",
+			gnutls_gost_paramset_get_name(paramset));
+
+	if (k) {
+		reverse_datum(k);
+		print_head(outfile, "private key", k->size, cprint);
+		print_hex_datum(outfile, k, cprint);
+	}
+	reverse_datum(x);
+	reverse_datum(y);
 	print_head(outfile, "x", x->size, cprint);
 	print_hex_datum(outfile, x, cprint);
 	print_head(outfile, "y", y->size, cprint);
 	print_hex_datum(outfile, y, cprint);
 }
 
-
 void
 print_rsa_pkey(FILE * outfile, gnutls_datum_t * m, gnutls_datum_t * e,
 	       gnutls_datum_t * d, gnutls_datum_t * p, gnutls_datum_t * q,
@@ -910,36 +993,40 @@ print_rsa_pkey(FILE * outfile, gnutls_datum_t * m, gnutls_datum_t * e,
 	}
 }
 
-void _pubkey_info(FILE * outfile,
-		  gnutls_certificate_print_formats_t format,
-		  gnutls_pubkey_t pubkey)
+void print_pubkey_info(gnutls_pubkey_t pubkey,
+		       FILE *outfile,
+		       gnutls_certificate_print_formats_t format,
+		       gnutls_x509_crt_fmt_t outcert_format,
+		       unsigned int outtext)
 {
 	gnutls_datum_t data;
 	int ret;
 	size_t size;
 
-	fix_lbuffer(0);
+	if (outtext) {
+		ret = gnutls_pubkey_print(pubkey, format, &data);
+		if (ret < 0) {
+			fprintf(stderr, "pubkey_print error: %s\n",
+				gnutls_strerror(ret));
+			app_exit(1);
+		}
 
-	ret = gnutls_pubkey_print(pubkey, format, &data);
-	if (ret < 0) {
-		fprintf(stderr, "pubkey_print error: %s\n",
-			gnutls_strerror(ret));
-		exit(1);
+		fprintf(outfile, "%s\n\n", data.data);
+		gnutls_free(data.data);
 	}
 
-	fprintf(outfile, "%s\n", data.data);
-	gnutls_free(data.data);
+	fix_lbuffer(0);
 
 	size = lbuffer_size;
 	ret =
-	    gnutls_pubkey_export(pubkey, GNUTLS_X509_FMT_PEM, lbuffer,
+	    gnutls_pubkey_export(pubkey, outcert_format, lbuffer,
 				 &size);
 	if (ret < 0) {
 		fprintf(stderr, "export error: %s\n", gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 
-	fprintf(outfile, "\n%s\n", lbuffer);
+	fwrite(lbuffer, 1, size, outfile);
 }
 
 static void
@@ -1000,12 +1087,17 @@ void dh_info(FILE * infile, FILE * outfile, common_info_st * ci)
 
 	if (gnutls_dh_params_init(&dh_params) < 0) {
 		fprintf(stderr, "Error in dh parameter initialization\n");
-		exit(1);
+		app_exit(1);
 	}
 
 	params.data = (void *) fread_file(infile, &size);
 	params.size = size;
 
+	if (params.data == NULL) {
+		fprintf(stderr, "Could not read input\n");
+		app_exit(1);
+	}
+
 	ret =
 	    gnutls_dh_params_import_pkcs3(dh_params, &params,
 					  ci->incert_format);
@@ -1015,7 +1107,7 @@ void dh_info(FILE * infile, FILE * outfile, common_info_st * ci)
 		if (ret2 < 0) {
 			fprintf(stderr, "Error parsing dh params: %s\n",
 				gnutls_strerror(ret));
-			exit(1);
+			app_exit(1);
 		}
 	}
 
@@ -1023,10 +1115,10 @@ void dh_info(FILE * infile, FILE * outfile, common_info_st * ci)
 	if (ret < 0) {
 		fprintf(stderr, "Error exporting parameters: %s\n",
 			gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 
-	if (ci->outcert_format == GNUTLS_X509_FMT_PEM)
+	if (ci->outtext)
 		print_dh_info(outfile, &p, &g, q_bits, ci->cprint);
 
 	if (!ci->cprint) {	/* generate a PKCS#3 structure */
@@ -1076,10 +1168,22 @@ int cipher_to_flags(const char *cipher)
 		return GNUTLS_PKCS_USE_PBES2_AES_256;
 	} else if (strcasecmp(cipher, "rc2-40") == 0) {
 		return GNUTLS_PKCS_USE_PKCS12_RC2_40;
+	} else if (strcasecmp(cipher, "gost28147-tc26z") == 0) {
+		return GNUTLS_PKCS_USE_PBES2_GOST_TC26Z;
+	} else if (strcasecmp(cipher, "gost28147-cpa") == 0) {
+		return GNUTLS_PKCS_USE_PBES2_GOST_CPA;
+	} else if (strcasecmp(cipher, "gost28147-cpb") == 0) {
+		return GNUTLS_PKCS_USE_PBES2_GOST_CPB;
+	} else if (strcasecmp(cipher, "gost28147-cpc") == 0) {
+		return GNUTLS_PKCS_USE_PBES2_GOST_CPC;
+	} else if (strcasecmp(cipher, "gost28147-cpd") == 0) {
+		return GNUTLS_PKCS_USE_PBES2_GOST_CPD;
+	} else if (strcasecmp(cipher, "none") == 0) {
+		return GNUTLS_PKCS_PLAIN;
 	}
 
 	fprintf(stderr, "unknown cipher %s\n", cipher);
-	exit(1);
+	app_exit(1);
 }
 
 static void privkey_info_int(FILE *outfile, common_info_st * cinfo,
@@ -1089,24 +1193,57 @@ static void privkey_info_int(FILE *outfile, common_info_st * cinfo,
 	unsigned int bits = 0;
 	size_t size;
 	const char *cprint;
+	gnutls_x509_spki_t spki;
+	gnutls_digest_algorithm_t dig;
 
 	/* Public key algorithm
 	 */
 	fprintf(outfile, "Public Key Info:\n");
-	ret = gnutls_x509_privkey_get_pk_algorithm2(key, &bits);
+	ret = gnutls_x509_spki_init(&spki);
+	if (ret < 0) {
+		fprintf(stderr, "spki_init: %s\n", gnutls_strerror(ret));
+		return;
+	}
+
 	fprintf(outfile, "\tPublic Key Algorithm: ");
 
-	key_type = ret;
+	key_type = gnutls_x509_privkey_get_pk_algorithm2(key, &bits);
 
 	cprint = gnutls_pk_algorithm_get_name(key_type);
 	fprintf(outfile, "%s\n", cprint ? cprint : "Unknown");
+
+	if (key_type == GNUTLS_PK_RSA_PSS) {
+		unsigned int salt_size;
+
+		ret = gnutls_x509_privkey_get_spki(key, spki, 0);
+		if (ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE)
+			goto spki_skip;
+
+		if (ret < 0) {
+			fprintf(stderr, "spki_get: %s\n", gnutls_strerror(ret));
+			goto spki_skip;
+		}
+
+		ret = gnutls_x509_spki_get_rsa_pss_params(spki, &dig, &salt_size);
+		if (ret < 0) {
+			fprintf(stderr, "spki_get_rsa_pss_params: %s\n",
+				gnutls_strerror(ret));
+		} else {
+			fprintf(outfile, "\t\tHash Algorithm: %s\n",
+				gnutls_digest_get_name(dig));
+			fprintf(outfile, "\t\tSalt Length: %d\n", salt_size);
+		}
+	}
+
+ spki_skip:
+	gnutls_x509_spki_deinit(spki);
 	fprintf(outfile, "\tKey Security Level: %s (%u bits)\n\n",
 		gnutls_sec_param_get_name(gnutls_x509_privkey_sec_param
 					  (key)), bits);
 
 	/* Print the raw public and private keys
 	 */
-	if (key_type == GNUTLS_PK_RSA) {
+	if (GNUTLS_PK_IS_RSA(key_type)) {
 		gnutls_datum_t m, e, d, p, q, u, exp1, exp2;
 
 		ret =
@@ -1150,7 +1287,7 @@ static void privkey_info_int(FILE *outfile, common_info_st * cinfo,
 			gnutls_free(q.data);
 			gnutls_free(g.data);
 		}
-	} else if (key_type == GNUTLS_PK_EC) {
+	} else if (key_type == GNUTLS_PK_ECDSA || key_type == GNUTLS_PK_EDDSA_ED25519) {
 		gnutls_datum_t y, x, k;
 		gnutls_ecc_curve_t curve;
 
@@ -1162,9 +1299,6 @@ static void privkey_info_int(FILE *outfile, common_info_st * cinfo,
 				"Error in key ECC data export: %s\n",
 				gnutls_strerror(ret));
 		else {
-			cprint = gnutls_ecc_curve_get_name(curve);
-			bits = 0;
-
 			print_ecc_pkey(outfile, curve, &k, &x, &y,
 				       cinfo->cprint);
 
@@ -1172,17 +1306,46 @@ static void privkey_info_int(FILE *outfile, common_info_st * cinfo,
 			gnutls_free(y.data);
 			gnutls_free(k.data);
 		}
+	} else if (key_type == GNUTLS_PK_GOST_01 ||
+		   key_type == GNUTLS_PK_GOST_12_256 ||
+		   key_type == GNUTLS_PK_GOST_12_512) {
+		gnutls_datum_t y, x, k;
+		gnutls_ecc_curve_t curve;
+		gnutls_digest_algorithm_t digest;
+		gnutls_gost_paramset_t paramset;
+
+		ret =
+		    gnutls_x509_privkey_export_gost_raw(key, &curve,
+							&digest,
+							&paramset,
+							&x, &y, &k);
+		if (ret < 0)
+			fprintf(stderr,
+				"Error in key GOST data export: %s\n",
+				gnutls_strerror(ret));
+		else {
+			print_gost_pkey(outfile, curve, digest, paramset,
+					&k, &x, &y,
+					cinfo->cprint);
+
+			gnutls_free(x.data);
+			gnutls_free(y.data);
+			gnutls_free(k.data);
+		}
+	}
+
+	size = lbuffer_size;
+	ret = gnutls_x509_privkey_get_seed(key, &dig, lbuffer, &size);
+	if (ret >= 0) {
+		fprintf(outfile, "Validation parameters:\n");
+		fprintf(outfile, "\tHash: %s\n",
+			gnutls_digest_get_name(dig));
+		fprintf(outfile, "\tSeed: %s\n",
+			raw_to_hex(lbuffer, size));
 	}
 
 	fprintf(outfile, "\n");
 
-	size = lbuffer_size;
-	ret = gnutls_x509_privkey_get_seed(key, NULL, lbuffer, &size);
-	if (ret >= 0) {
-		fprintf(outfile, "Seed: %s\n",
-			raw_to_string(lbuffer, size));
-	}
-
 	size = lbuffer_size;
 	ret =
 	     gnutls_x509_privkey_get_key_id(key, GNUTLS_KEYID_USE_SHA256, lbuffer, &size);
@@ -1213,21 +1376,18 @@ print_private_key(FILE *outfile, common_info_st * cinfo, gnutls_x509_privkey_t k
 {
 	int ret;
 	size_t size;
-	unsigned int flags = 0;
 
 	if (!key)
 		return;
 
-	if (!cinfo->pkcs8 && gnutls_x509_privkey_get_seed(key, NULL, NULL, 0) != GNUTLS_E_INVALID_REQUEST) {
-		cinfo->pkcs8 = 1;
-		flags |= GNUTLS_PKCS_PLAIN;
-	}
+	/* Only print private key parameters when an unencrypted
+	 * format is used */
+	if (cinfo->outtext)
+		privkey_info_int(outfile, cinfo, key);
+
+	switch_to_pkcs8_when_needed(cinfo, key, gnutls_x509_privkey_get_pk_algorithm(key));
 
 	if (!cinfo->pkcs8) {
-		/* Only print private key parameters when an unencrypted
-		 * format is used */
-		if (cinfo->outcert_format == GNUTLS_X509_FMT_PEM)
-			privkey_info_int(outfile, cinfo, key);
 
 		size = lbuffer_size;
 		ret = gnutls_x509_privkey_export(key, cinfo->outcert_format,
@@ -1235,18 +1395,14 @@ print_private_key(FILE *outfile, common_info_st * cinfo, gnutls_x509_privkey_t k
 		if (ret < 0) {
 			fprintf(stderr, "privkey_export: %s\n",
 				gnutls_strerror(ret));
-			exit(1);
+			app_exit(1);
 		}
 	} else {
-		const char *pass = NULL;
+		unsigned int flags = 0;
+		const char *pass;
 
-		if (!(flags & GNUTLS_PKCS_PLAIN)) {
-			pass = get_password(cinfo, &flags, 0);
-			flags |= cipher_to_flags(cinfo->pkcs_cipher);
-		} else {
-			if (cinfo->outcert_format == GNUTLS_X509_FMT_PEM)
-				privkey_info_int(outfile, cinfo, key);
-		}
+		pass = get_password(cinfo, &flags, 0);
+		flags |= cipher_to_flags(cinfo->pkcs_cipher);
 
 		size = lbuffer_size;
 		ret =
@@ -1256,7 +1412,7 @@ print_private_key(FILE *outfile, common_info_st * cinfo, gnutls_x509_privkey_t k
 		if (ret < 0) {
 			fprintf(stderr, "privkey_export_pkcs8: %s\n",
 				gnutls_strerror(ret));
-			exit(1);
+			app_exit(1);
 		}
 	}
 
@@ -1293,7 +1449,7 @@ int generate_prime(FILE * outfile, int how, common_info_st * info)
 				fprintf(stderr,
 					"Error initializing key: %s\n",
 					gnutls_strerror(ret));
-				exit(1);
+				app_exit(1);
 			}
 
 			if (info->seed_size > 0) {
@@ -1301,7 +1457,7 @@ int generate_prime(FILE * outfile, int how, common_info_st * info)
 
 				if (info->seed_size < 32) {
 					fprintf(stderr, "For DH parameter generation a 32-byte seed value or larger is expected (have: %d); use -d 2 for more information.\n", (int)info->seed_size);
-					exit(1);
+					app_exit(1);
 				}
 
 				data.type = GNUTLS_KEYGEN_SEED;
@@ -1317,7 +1473,7 @@ int generate_prime(FILE * outfile, int how, common_info_st * info)
 				fprintf(stderr,
 					"Error generating DSA parameters: %s\n",
 					gnutls_strerror(ret));
-				exit(1);
+				app_exit(1);
 			}
 
 			if (info->outcert_format == GNUTLS_X509_FMT_PEM) {
@@ -1329,7 +1485,7 @@ int generate_prime(FILE * outfile, int how, common_info_st * info)
 				fprintf(stderr,
 					"Error importing DSA parameters: %s\n",
 					gnutls_strerror(ret));
-				exit(1);
+				app_exit(1);
 			}
 
 			gnutls_x509_privkey_deinit(pkey);
@@ -1339,7 +1495,7 @@ int generate_prime(FILE * outfile, int how, common_info_st * info)
 				fprintf(stderr,
 					"Error generating parameters: %s\n",
 					gnutls_strerror(ret));
-				exit(1);
+				app_exit(1);
 			}
 		}
 
@@ -1349,79 +1505,74 @@ int generate_prime(FILE * outfile, int how, common_info_st * info)
 		if (ret < 0) {
 			fprintf(stderr, "Error exporting parameters: %s\n",
 				gnutls_strerror(ret));
-			exit(1);
+			app_exit(1);
 		}
 	} else {
 		if (info->provable != 0) {
 			fprintf(stderr, "The DH parameters obtained via this option are not provable\n");
-			exit(1);
+			app_exit(1);
 		}
 #if defined(ENABLE_DHE) || defined(ENABLE_ANON)
 		if (bits <= 2048) {
 			p = gnutls_ffdhe_2048_group_prime;
 			g = gnutls_ffdhe_2048_group_generator;
 			key_bits = gnutls_ffdhe_2048_key_bits;
-			bits = 2048;
 		} else if (bits <= 3072) {
 			p = gnutls_ffdhe_3072_group_prime;
 			g = gnutls_ffdhe_3072_group_generator;
 			key_bits = gnutls_ffdhe_3072_key_bits;
-			bits = 3072;
 		} else if (bits <= 4096) {
 			p = gnutls_ffdhe_4096_group_prime;
 			g = gnutls_ffdhe_4096_group_generator;
 			key_bits = gnutls_ffdhe_4096_key_bits;
-			bits = 4096;
+		} else if (bits <= 6144) {
+			p = gnutls_ffdhe_6144_group_prime;
+			g = gnutls_ffdhe_6144_group_generator;
+			key_bits = gnutls_ffdhe_6144_key_bits;
 		} else {
 			p = gnutls_ffdhe_8192_group_prime;
 			g = gnutls_ffdhe_8192_group_generator;
 			key_bits = gnutls_ffdhe_8192_key_bits;
-			bits = 8192;
 		}
 
 		ret = gnutls_dh_params_import_raw2(dh_params, &p, &g, key_bits);
 		if (ret < 0) {
 			fprintf(stderr, "Error exporting parameters: %s\n",
 				gnutls_strerror(ret));
-			exit(1);
+			app_exit(1);
 		}
 #elif defined(ENABLE_SRP)
 		if (bits <= 1024) {
 			p = gnutls_srp_1024_group_prime;
 			g = gnutls_srp_1024_group_generator;
-			bits = 1024;
 		} else if (bits <= 1536) {
 			p = gnutls_srp_1536_group_prime;
 			g = gnutls_srp_1536_group_generator;
-			bits = 1536;
 		} else if (bits <= 2048) {
 			p = gnutls_srp_2048_group_prime;
 			g = gnutls_srp_2048_group_generator;
-			bits = 2048;
 		} else if (bits <= 3072) {
 			p = gnutls_srp_3072_group_prime;
 			g = gnutls_srp_3072_group_generator;
-			bits = 3072;
 		} else {
 			p = gnutls_srp_4096_group_prime;
 			g = gnutls_srp_4096_group_generator;
-			bits = 4096;
 		}
 
 		ret = gnutls_dh_params_import_raw(dh_params, &p, &g);
 		if (ret < 0) {
 			fprintf(stderr, "Error exporting parameters: %s\n",
 				gnutls_strerror(ret));
-			exit(1);
+			app_exit(1);
 		}
 #else
 		fprintf(stderr,
 			"Parameters unavailable as SRP is disabled.\n");
-		exit(1);
+		app_exit(1);
 #endif
 	}
 
-	if (info->outcert_format == GNUTLS_X509_FMT_PEM)
+	if (info->outtext)
 		print_dh_info(outfile, &p, &g, q_bits, info->cprint);
 
 	if (!info->cprint) {	/* generate a PKCS#3 structure */
@@ -1465,16 +1616,97 @@ void decode_seed(gnutls_datum_t *seed, const char *hex, unsigned hex_size)
 
 	if (seed->data == NULL) {
 		fprintf(stderr, "memory error\n");
-		exit(1);
+		app_exit(1);
 	}
 
 	seed_size = hex_size;
 	ret = gnutls_hex2bin(hex, hex_size, seed->data, &seed_size);
 	if (ret < 0) {
 		fprintf(stderr, "Could not hex decode data: %s\n", gnutls_strerror(ret));
-		exit(1);
+		app_exit(1);
 	}
 	seed->size = seed_size;
 
 	return;
 }
+
+gnutls_pk_algorithm_t figure_key_type(const char *key_type)
+{
+	if (strcasecmp(key_type, "rsa") == 0)
+		return GNUTLS_PK_RSA;
+	else if (strcasecmp(key_type, "rsa-pss") == 0)
+		return GNUTLS_PK_RSA_PSS;
+	else if (strcasecmp(key_type, "ed25519") == 0 || strcasecmp(key_type, "eddsa") == 0)
+		return GNUTLS_PK_EDDSA_ED25519;
+	else if (strcasecmp(key_type, "dsa") == 0)
+		return GNUTLS_PK_DSA;
+	else if (strcasecmp(key_type, "ecdsa") == 0 || strcasecmp(key_type, "ecc") == 0)
+		return GNUTLS_PK_ECDSA;
+	else if (strcasecmp(key_type, "gost01") == 0)
+		return GNUTLS_PK_GOST_01;
+	else if (strcasecmp(key_type, "gost12-256") == 0)
+		return GNUTLS_PK_GOST_12_256;
+	else if (strcasecmp(key_type, "gost12-512") == 0)
+		return GNUTLS_PK_GOST_12_512;
+	else {
+		fprintf(stderr, "unknown key type: %s\n", key_type);
+		return GNUTLS_PK_UNKNOWN;
+	}
+}
+
+gnutls_digest_algorithm_t hash_to_id(const char *hash)
+{
+	if (strcasecmp(hash, "md5") == 0) {
+		fprintf(stderr,
+			"Warning: MD5 is broken, and should not be used any more for digital signatures.\n");
+		return GNUTLS_DIG_MD5;
+	} else if (strcasecmp(hash, "sha1") == 0)
+		return GNUTLS_DIG_SHA1;
+	else if (strcasecmp(hash, "sha256") == 0)
+		return GNUTLS_DIG_SHA256;
+	else if (strcasecmp(hash, "sha224") == 0)
+		return GNUTLS_DIG_SHA224;
+	else if (strcasecmp(hash, "sha384") == 0)
+		return GNUTLS_DIG_SHA384;
+	else if (strcasecmp(hash, "sha512") == 0)
+		return GNUTLS_DIG_SHA512;
+	else if (strcasecmp(hash, "sha3-256") == 0)
+		return GNUTLS_DIG_SHA3_256;
+	else if (strcasecmp(hash, "sha3-224") == 0)
+		return GNUTLS_DIG_SHA3_224;
+	else if (strcasecmp(hash, "sha3-384") == 0)
+		return GNUTLS_DIG_SHA3_384;
+	else if (strcasecmp(hash, "sha3-512") == 0)
+		return GNUTLS_DIG_SHA3_512;
+	else if (strcasecmp(hash, "rmd160") == 0)
+		return GNUTLS_DIG_RMD160;
+	else {
+		return gnutls_digest_get_id(hash);
+	}
+}
+
+void sign_params_to_flags(common_info_st *cinfo, const char *params)
+{
+	char *p, *sp;
+
+	sp = strdup(params);
+	if (sp == NULL) {
+		fprintf(stderr, "memory error\n");
+		app_exit(1);
+	}
+
+	p = strtok(sp, ",");
+
+	while(p != NULL) {
+		if (strcasecmp(p, "rsa-pss")==0) {
+			cinfo->rsa_pss_sign = 1;
+		} else {
+			fprintf(stderr, "Unknown signature parameter: %s\n", p);
+			app_exit(1);
+		}
+
+		p=strtok(NULL, ",");
+	}
+
+	free(sp);
+}
