@@ -15,13 +15,17 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
+#ifndef INCLUDE_COMMON_H
+# define INCLUDE_COMMON_H
+
 #define SERVER "127.0.0.1"
 
 #include <config.h>
 #include <gnutls/gnutls.h>
+#include <gnutls/pkcs11.h>
 #include <certtool-common.h>
 #include <c-ctype.h>
 #include <string.h>
@@ -56,7 +60,9 @@ extern const char str_unknown[];
 #define P_WAIT_FOR_CERT (1<<1)
 int print_info(gnutls_session_t state, int verbose, int flags);
 void print_cert_info(gnutls_session_t, int flag, int print_cert);
-void print_cert_info_compact(gnutls_session_t session);
+
+int log_msg(FILE *file, const char *message, ...) __attribute__((format(printf, 2, 3)));
+void log_set(FILE *file);
 
 void print_cert_info2(gnutls_session_t, int flag, FILE *fp, int print_cert);
 
@@ -64,7 +70,13 @@ void print_list(const char *priorities, int verbose);
 int cert_verify(gnutls_session_t session, const char *hostname, const char *purpose);
 
 const char *raw_to_string(const unsigned char *raw, size_t raw_size);
-int check_command(gnutls_session_t session, const char *str);
+const char *raw_to_hex(const unsigned char *raw, size_t raw_size);
+const char *raw_to_base64(const unsigned char *raw, size_t raw_size);
+int check_command(gnutls_session_t session, const char *str, unsigned no_cli_cert);
+
+#define MAX_PIN_LEN GNUTLS_PKCS11_MAX_PIN_LEN
+void getenv_copy(char *str, size_t max_str_size, const char *envvar);
+void getpass_copy(char *pass, size_t max_pass_size, const char *prompt);
 
 int
 pin_callback(void *user, int attempt, const char *token_url,
@@ -84,6 +96,8 @@ int len = strlen(hostname);
 	return 0;
 }
 
+void sockets_init(void);
+
 #ifdef _WIN32
 static int system_recv_timeout(gnutls_transport_ptr_t ptr, unsigned int ms)
 {
@@ -94,12 +108,8 @@ static int system_recv_timeout(gnutls_transport_ptr_t ptr, unsigned int ms)
 	FD_ZERO(&rfds);
 	FD_SET(fd, &rfds);
 
-	tv.tv_sec = 0;
-	tv.tv_usec = ms * 1000;
-	while (tv.tv_usec >= 1000000) {
-		tv.tv_usec -= 1000000;
-		tv.tv_sec++;
-	}
+	tv.tv_sec = ms / 1000;
+	tv.tv_usec = (ms % 1000) * 1000;
 
 	return select(fd + 1, &rfds, NULL, NULL, &tv);
 }
@@ -126,3 +136,5 @@ void set_read_funcs(gnutls_session_t session)
 #else
 # define set_read_funcs(x)
 #endif
+
+#endif
