@@ -5,6 +5,9 @@
 #define SOCKET_FLAG_FASTOPEN (1<<1)
 #define SOCKET_FLAG_STARTTLS (1<<2)
 #define SOCKET_FLAG_RAW (1<<3) /* unencrypted */
+#define SOCKET_FLAG_VERBOSE (1<<4)
+#define SOCKET_FLAG_SKIP_INIT (1<<5)
+#define SOCKET_FLAG_DONT_PRINT_ERRORS (1<<6)
 
 
 typedef struct {
@@ -23,8 +26,13 @@ typedef struct {
 	struct sockaddr_storage connect_addr;
 	socklen_t connect_addrlen;
 
+	FILE *server_trace;
+	FILE *client_trace;
+
 	/* resumption data */
 	gnutls_datum_t rdata;
+	/* early data */
+	gnutls_datum_t edata;
 } socket_st;
 
 /* calling program must provide that */
@@ -40,10 +48,17 @@ ssize_t socket_send(const socket_st * socket, const void *buffer,
 ssize_t socket_send_range(const socket_st * socket, const void *buffer,
 			  int buffer_size, gnutls_range_st * range);
 void
-socket_open(socket_st * hd, const char *hostname, const char *service,
-	    const char *app_proto, int flags, const char *msg, gnutls_datum_t *rdata);
+socket_open2(socket_st * hd, const char *hostname, const char *service,
+	    const char *app_proto, int flags, const char *msg, gnutls_datum_t *rdata, gnutls_datum_t *edata,
+	    FILE *server_trace, FILE *client_trace);
 
-void socket_bye(socket_st * socket);
+#define socket_open(hd, host, service, app_proto, flags, msg, rdata) \
+	socket_open2(hd, host, service, app_proto, flags, msg, rdata, NULL, NULL, NULL)
+
+#define socket_open3(hd, host, service, app_proto, flags, msg, rdata, edata)	\
+	socket_open2(hd, host, service, app_proto, flags, msg, rdata, edata, NULL, NULL)
+
+void socket_bye(socket_st * socket, unsigned polite);
 
 int service_to_port(const char *service, const char *proto);
 const char *port_to_service(const char *sport, const char *proto);
