@@ -15,11 +15,11 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 typedef enum {
-	TEST_SUCCEED, TEST_FAILED, TEST_UNSURE, TEST_IGNORE
+	TEST_SUCCEED, TEST_FAILED, TEST_UNSURE, TEST_IGNORE/*keep socket*/, TEST_IGNORE2/*destroy socket*/
 } test_code_t;
 
 test_code_t test_chain_order(gnutls_session_t session);
@@ -30,6 +30,7 @@ test_code_t test_heartbeat_extension(gnutls_session_t state);
 test_code_t test_small_records(gnutls_session_t state);
 test_code_t test_rfc7507(gnutls_session_t state);
 test_code_t test_dhe(gnutls_session_t state);
+test_code_t test_rfc7919(gnutls_session_t state);
 test_code_t test_dhe_group(gnutls_session_t state);
 test_code_t test_ssl3(gnutls_session_t state);
 test_code_t test_aes(gnutls_session_t state);
@@ -39,6 +40,7 @@ test_code_t test_md5(gnutls_session_t state);
 test_code_t test_sha(gnutls_session_t state);
 test_code_t test_3des(gnutls_session_t state);
 test_code_t test_arcfour(gnutls_session_t state);
+test_code_t test_chacha20(gnutls_session_t state);
 test_code_t test_tls1(gnutls_session_t state);
 test_code_t test_tls1_nossl3(gnutls_session_t session);
 test_code_t test_safe_renegotiation(gnutls_session_t state);
@@ -47,6 +49,7 @@ test_code_t test_etm(gnutls_session_t state);
 test_code_t test_safe_renegotiation_scsv(gnutls_session_t state);
 test_code_t test_tls1_1(gnutls_session_t state);
 test_code_t test_tls1_2(gnutls_session_t state);
+test_code_t test_tls1_3(gnutls_session_t state);
 test_code_t test_tls1_1_fallback(gnutls_session_t state);
 test_code_t test_tls1_6_fallback(gnutls_session_t state);
 test_code_t test_tls_disable0(gnutls_session_t state);
@@ -58,18 +61,19 @@ test_code_t test_max_record_size(gnutls_session_t state);
 test_code_t test_version_rollback(gnutls_session_t state);
 test_code_t test_anonymous(gnutls_session_t state);
 test_code_t test_unknown_ciphersuites(gnutls_session_t state);
-test_code_t test_openpgp1(gnutls_session_t state);
 test_code_t test_bye(gnutls_session_t state);
 test_code_t test_certificate(gnutls_session_t state);
 test_code_t test_server_cas(gnutls_session_t state);
 test_code_t test_session_resume2(gnutls_session_t state);
 test_code_t test_rsa_pms_version_check(gnutls_session_t session);
 test_code_t test_version_oob(gnutls_session_t session);
-test_code_t test_zlib(gnutls_session_t session);
 int _test_srp_username_callback(gnutls_session_t session,
 				char **username, char **password);
 
-test_code_t test_ecdhe_curve(gnutls_session_t session);
+test_code_t test_ecdhe_x25519(gnutls_session_t session);
+test_code_t test_ecdhe_secp521r1(gnutls_session_t session);
+test_code_t test_ecdhe_secp384r1(gnutls_session_t session);
+test_code_t test_ecdhe_secp256r1(gnutls_session_t session);
 test_code_t test_ecdhe(gnutls_session_t session);
 test_code_t test_aes_gcm(gnutls_session_t session);
 test_code_t test_aes_ccm(gnutls_session_t session);
