@@ -59,7 +59,10 @@ check_if_port_listening() {
 }
 
 # Find a port number not currently in use.
-GETPORT='rc=0; myrandom=$(date +%N | sed s/^0*//)
+GETPORT='rc=0; unset myrandom
+    if test -n "$RANDOM"; then myrandom=$(($RANDOM + $RANDOM)); fi
+    if test -z "$myrandom"; then myrandom=$(date +%N | sed s/^0*//); fi
+    if test -z "$myrandom"; then myrandom=0; fi
     while test $rc = 0;do
 	PORT="$(((($$<<15)|$myrandom) % 63001 + 2000))"
 	check_if_port_in_use $PORT;rc=$?
@@ -67,6 +70,12 @@ GETPORT='rc=0; myrandom=$(date +%N | sed s/^0*//)
 '
 
 check_for_datefudge() {
+	# On certain platforms running datefudge date fails (e.g., x86 datefudge
+	# with x86-64 date app).
+	if test "${SKIP_DATEFUDGE_CHECK}" = 1;then
+		return
+	fi
+
 	TSTAMP=`datefudge -s "2006-09-23" date -u +%s || true`
 	if test "$TSTAMP" != "1158969600" || test "$WINDOWS" = 1; then
 	echo $TSTAMP
@@ -95,11 +104,23 @@ if test $? = 0;then
 fi
 }
 
+exit_if_non_padlock()
+{
+which lscpu >/dev/null 2>&1
+if test $? = 0;then
+        $(which lscpu)|grep Flags|grep phe
+        if test $? != 0;then
+                echo "non-Via padlock CPU detected"
+                exit 0
+        fi
+fi
+}
+
 wait_for_port()
 {
 	local ret
 	local PORT="$1"
-	sleep 4
+	sleep 1
 
 	for i in 1 2 3 4 5 6;do
 		check_if_port_listening ${PORT}
@@ -126,7 +147,7 @@ wait_for_free_port()
 		if test $ret != 0;then
 			break
 		else
-			sleep 20
+			sleep 2
 		fi
 	done
 	return $ret
@@ -137,7 +158,7 @@ launch_server() {
 	shift
 
 	wait_for_free_port ${PORT}
-	${SERV} ${DEBUG} -p "${PORT}" $* >/dev/null 2>&1 &
+	${SERV} ${DEBUG} -p "${PORT}" $* >/dev/null &
 }
 
 launch_pkcs11_server() {
@@ -156,7 +177,7 @@ launch_bare_server() {
 	shift
 
 	wait_for_free_port ${PORT}
-	${SERV} $* >/dev/null 2>&1 &
+	${SERV} $* >/dev/null &
 }
 
 wait_server() {
@@ -176,3 +197,87 @@ wait_udp_server() {
 	sleep 4
 }
 
+if test -x /usr/bin/lockfile-create;then
+LOCKFILE="lockfile-create global"
+UNLOCKFILE="lockfile-remove global"
+else
+LOCKFILE="lockfile global.lock"
+UNLOCKFILE="rm -f global.lock"
+fi
+
+create_testdir() {
+	local PREFIX=$1
+	d=`mktemp -d -t ${PREFIX}.XXXXXX`
+	if test $? -ne 0; then
+		d=${TMPDIR}/${PREFIX}.$$
+		mkdir "$d" || exit 1
+	fi
+	trap "test -e \"$d\" && rm -rf \"$d\"" 1 15 2
+	echo "$d"
+}
+
+wait_for_file() {
+	local filename="$1"
+	local timeout="$2"
+
+	local loops=$((timeout * 10)) loop=0
+
+	while test $loop -lt $loops; do
+		[ -f "$filename" ] && {
+			#allow file to be written to
+			sleep 0.2
+			return 1
+		}
+		sleep 0.1
+		loop=$((loop+1))
+	done
+	return 0
+}
+
+# Kill a process quietly
+# @1: signal, e.g. -9
+# @2: pid
+kill_quiet() {
+	local sig="$1"
+	local pid="$2"
+
+	sh -c "kill $sig $pid 2>/dev/null"
+	return $?
+}
+
+# Terminate a process first using SIGTERM, wait 1s and if still avive use
+# SIGKILL
+# @1: pid
+terminate_proc() {
+	local pid="$1"
+
+	local ctr=0
+
+	kill_quiet -15 $pid
+	while [ $ctr -lt 10 ]; do
+		sleep 0.1
+		kill -0 $pid 2>/dev/null
+		[ $? -ne 0 ] && return
+		ctr=$((ctr + 1))
+	done
+	kill_quiet -9 $pid
+	sleep 0.1
+}
+
+# $1, $2: the two files to check for equality
+# $3: Strings to be ignored, separated by |
+check_if_equal() {
+	if test -n "$3"; then
+		local tmp1=`basename "$1"`"1.tmp"
+		local tmp2=`basename "$2"`"2.tmp"
+		egrep -v "$3" "$1" | tr -d '\r' >"$tmp1"
+		egrep -v "$3" "$2" | tr -d '\r' >"$tmp2"
+		diff -b -B "$tmp1" "$tmp2"
+		local rc=$?
+		rm -f "$tmp1" "$tmp2"
+		return $rc
+	fi
+
+	diff -b -B "$1" "$2"
+	return $?
+}
