@@ -16,7 +16,8 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with GnuTLS.  If not, see <https://www.gnu.org/licenses/>.
+ * along with GnuTLS; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -27,7 +28,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <gnutls/gnutls.h>
-#include <assert.h>
 
 #include "utils.h"
 
@@ -36,9 +36,10 @@
  * variable to specify the test file (system.prio).
  */
 
-char *_gnutls_resolve_priorities(const char *priorities);
+char *_gnutls_resolve_priorities(const char* priorities);
 
-static void try_prio(const char *prio, const char *expected_str)
+static void
+try_prio(const char *prio, const char *expected_str)
 {
 	char *p;
 
@@ -50,15 +51,12 @@ static void try_prio(const char *prio, const char *expected_str)
 	if (p == NULL && expected_str == NULL)
 		goto ok;
 
-	assert(strstr(gnutls_get_system_config_file(), "system.prio") != NULL);
-
 	if (p == NULL || expected_str == NULL || strcmp(p, expected_str) != 0) {
-		fail("test: %s: error; got: %s, expected: %s\n", prio, p,
-		     expected_str);
+		fail("test: %s: error; got: %s, expected: %s\n", prio, p, expected_str);
 		exit(1);
 	}
 
-ok:
+ ok:
 	gnutls_free(p);
 	gnutls_global_deinit();
 }
@@ -70,9 +68,7 @@ void doit(void)
 	try_prio("@HELLO1", "NORMAL");
 	try_prio("@HELLO1:+AES-256-CBC:+AEAD", "NORMAL:+AES-256-CBC:+AEAD");
 	try_prio("@HELLO2", "NORMAL:+AES-128-CBC");
-	try_prio(
-		"@HELLO3",
-		"NONE:+VERS-TLS-ALL:-VERS-SSL3.0:+AEAD:+SHA1:+SHA256:+SHA384:+ECDHE-RSA:+ECDHE-ECDSA:+RSA:+DHE-RSA:+DHE-DSS:+AES-256-GCM:+AES-256-CBC:+CAMELLIA-256-GCM:+CAMELLIA-256-CBC:+AES-128-GCM:+AES-128-CBC:+CAMELLIA-128-GCM:+CAMELLIA-128-CBC:+3DES-CBC:+SIGN-ALL:-SIGN-RSA-MD5:+CURVE-ALL:+COMP-NULL:%PROFILE_LOW");
+	try_prio("@HELLO3", "NONE:+VERS-TLS-ALL:-VERS-SSL3.0:+AEAD:+SHA1:+SHA256:+SHA384:+ECDHE-RSA:+ECDHE-ECDSA:+RSA:+DHE-RSA:+DHE-DSS:+AES-256-GCM:+AES-256-CBC:+CAMELLIA-256-GCM:+CAMELLIA-256-CBC:+AES-128-GCM:+AES-128-CBC:+CAMELLIA-128-GCM:+CAMELLIA-128-CBC:+3DES-CBC:+SIGN-ALL:-SIGN-RSA-MD5:+CURVE-ALL:+COMP-NULL:%PROFILE_LOW");
 	try_prio("@HELLO1,HELLO2", "NORMAL");
 	try_prio("@HELLO1,HELLO2:+AES-128-CBC", "NORMAL:+AES-128-CBC");
 	try_prio("@HELLO1,HELLO1", "NORMAL");
@@ -82,8 +78,7 @@ void doit(void)
 	try_prio("@HELLO2,HELLO1,@HELLO3", "NORMAL:+AES-128-CBC");
 	try_prio("@HELLONO,HELLO1", "NORMAL");
 	try_prio("@HELLONO,HELLONO2,HELLO1", "NORMAL");
-	try_prio("@HELLONO,HELLONO2,HELLO1:+AES-128-CBC",
-		 "NORMAL:+AES-128-CBC");
+	try_prio("@HELLONO,HELLONO2,HELLO1:+AES-128-CBC", "NORMAL:+AES-128-CBC");
 	try_prio("@HELLONO", NULL);
 	try_prio("@HELLONO,", NULL);
 	try_prio("@HELLONO:+AES-128-CBC", NULL);
