@@ -18,7 +18,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 #include <stdio.h>
@@ -65,19 +65,14 @@ static void client_log_func(int level, const char *str)
 /* These are global */
 static pid_t child;
 
-static const gnutls_datum_t hrnd = {
-	(void *)"\x00\x02\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
-	32
-};
-
-static const gnutls_datum_t hsrnd = {
-	(void *)"\x00\x05\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
-	32
-};
+static const
+gnutls_datum_t hrnd = {(void*)"\x00\x02\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", 32};
+static const
+gnutls_datum_t hsrnd = {(void*)"\x00\x05\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", 32};
 
 static int gnutls_rnd_works;
 
-int __attribute__((visibility("protected")))
+int __attribute__ ((visibility ("protected")))
 gnutls_rnd(gnutls_rnd_level_t level, void *data, size_t len)
 {
 	gnutls_rnd_works = 1;
@@ -96,88 +91,74 @@ static void dump(const char *name, const uint8_t *data, unsigned data_size)
 	unsigned i;
 
 	fprintf(stderr, "%s", name);
-	for (i = 0; i < data_size; i++)
+	for (i=0;i<data_size;i++)
 		fprintf(stderr, "\\x%.2x", (unsigned)data[i]);
 	fprintf(stderr, "\n");
 }
 
-#define TRY(label_size, label, extra_size, extra, size, exp)                           \
-	{                                                                              \
-		ret = gnutls_prf_rfc5705(session, label_size, label,                   \
-					 extra_size, extra, size,                      \
-					 (void *)key_material);                        \
-		if (ret < 0) {                                                         \
-			fprintf(stderr, "gnutls_prf_rfc5705: error in %d\n",           \
-				__LINE__);                                             \
-			gnutls_perror(ret);                                            \
-			exit(1);                                                       \
-		}                                                                      \
-		if (memcmp(key_material, exp, size) != 0) {                            \
-			fprintf(stderr,                                                \
-				"gnutls_prf_rfc5705: output doesn't match for '%s'\n", \
-				label);                                                \
-			dump("got ", key_material, size);                              \
-			dump("expected ", exp, size);                                  \
-			exit(1);                                                       \
-		}                                                                      \
+#define TRY(label_size, label, extra_size, extra, size, exp) \
+	{ \
+	ret = gnutls_prf_rfc5705(session, label_size, label, extra_size, extra, size, \
+			 (void*)key_material); \
+	if (ret < 0) { \
+		fprintf(stderr, "gnutls_prf_rfc5705: error in %d\n", __LINE__); \
+		gnutls_perror(ret); \
+		exit(1); \
+	} \
+	if (memcmp(key_material, exp, size) != 0) { \
+		fprintf(stderr, "gnutls_prf_rfc5705: output doesn't match for '%s'\n", label); \
+		dump("got ", key_material, size); \
+		dump("expected ", exp, size); \
+		exit(1); \
+	} \
 	}
 
-#define TRY_OLD(label_size, label, size, exp)                                  \
-	{                                                                      \
-		ret = gnutls_prf(session, label_size, label, 0, 0, NULL, size, \
-				 (void *)key_material);                        \
-		if (ret < 0) {                                                 \
-			fprintf(stderr, "gnutls_prf: error in %d\n",           \
-				__LINE__);                                     \
-			gnutls_perror(ret);                                    \
-			exit(1);                                               \
-		}                                                              \
-		if (memcmp(key_material, exp, size) != 0) {                    \
-			fprintf(stderr,                                        \
-				"gnutls_prf: output doesn't match for '%s'\n", \
-				label);                                        \
-			dump("got ", key_material, size);                      \
-			dump("expected ", exp, size);                          \
-			exit(1);                                               \
-		}                                                              \
+#define TRY_OLD(label_size, label, size, exp) \
+	{ \
+	ret = gnutls_prf(session, label_size, label, 0, 0, NULL, size, \
+			 (void*)key_material); \
+	if (ret < 0) { \
+		fprintf(stderr, "gnutls_prf: error in %d\n", __LINE__); \
+		gnutls_perror(ret); \
+		exit(1); \
+	} \
+	if (memcmp(key_material, exp, size) != 0) { \
+		fprintf(stderr, "gnutls_prf: output doesn't match for '%s'\n", label); \
+		dump("got ", key_material, size); \
+		dump("expected ", exp, size); \
+		exit(1); \
+	} \
 	}
 
-#define KEY_EXP_VALUE \
-	"\x28\x70\xa8\x34\xd4\x43\x85\xfd\x55\xe0\x13\x78\x75\xa3\x25\xa7\xfd\x0b\x6b\x68\x5d\x62\x72\x02\xdf\x3d\x79\xca\x55\xab\xea\x24\xf3\x4d"
-#define HELLO_VALUE \
-	"\xd8\xcb\x72\x1e\x24\x2d\x79\x11\x41\x38\x05\x2b\x1b\x5d\x60\x12\x30\x0a\xf7\x1e\x23\x90\x4d\x64\xf8\xf5\x23\xea\xbf\xa3\x24"
-#define CONTEXT_VALUE \
-	"\xe6\xc0\x57\xbe\xda\x28\x9c\xc7\xf6\x4f\xb6\x18\x92\xce\x10\xf6\xe1\x5e\xab\x10\xc8\xd1\x94\xf8\xac\xc7\x3e\x93\xde\x57\x12"
-#define NULL_CONTEXT_VALUE \
-	"\xaf\xea\xd2\x64\xc9\x42\xbd\xe7\xdb\xf0\xd3\x16\x84\x39\xf3\xdb\x5d\x4f\x0e\x5e\x71\x1e\xc0\xd7\x23\xde\x8b\x1e\x80\xa1\xca"
+#define KEY_EXP_VALUE "\xec\x26\x9e\x8c\x5f\xff\x5c\xb2\x60\x4f\x82\xe7\x6b\xb9\x70\x40\xb9\x2d\x2f\xe7\x41\xa8\xe7\xfa\x03\x7c\xe8\x6d\xfa\xda\xc2\xa9\x3f\x58"
+#define HELLO_VALUE "\xd4\x74\x4a\x09\x28\x0a\x99\xb9\xa4\x5b\x51\x5b\x80\xe7\x50\x1c\x16\xca\x57\x78\xf0\xe5\xa1\x94\x6b\x20\x2b\x14\xff\x2b\x53"
+#define CONTEXT_VALUE "\x8d\xde\xea\x58\xab\x90\xaf\x6c\x5c\x7a\x69\xbf\x8a\xd2\x16\xb4\x0f\x75\xb8\x63\xdb\x86\xe7\x66\x04\x59\xac\x57\xe0\x03\x37"
+#define NULL_CONTEXT_VALUE "\x6c\x1a\x10\x1f\xa9\x5a\xfd\xcd\xf4\xcf\x27\x09\x00\xa8\xca\x8e\x8a\x56\xfb\x80\xf0\x0d\xb3\xa6\xe9\x4a\x5f\xe0\x0c\x31\xd9"
 static void check_prfs(gnutls_session_t session)
 {
 	unsigned char key_material[512];
 	int ret;
 
 	if (!gnutls_rnd_works) {
-		fprintf(stderr,
-			"gnutls_rnd() could not be overridden, see #584\n");
+		fprintf(stderr, "gnutls_rnd() could not be overridden, see #584\n");
 		exit(77);
 	}
 
-	TRY_OLD(13, "key expansion", 34, (uint8_t *)KEY_EXP_VALUE);
-	TRY_OLD(6, "hello", 31, (uint8_t *)HELLO_VALUE);
+	TRY_OLD(13, "key expansion", 34, (uint8_t*)KEY_EXP_VALUE);
+	TRY_OLD(6, "hello", 31, (uint8_t*)HELLO_VALUE);
 
-	TRY(13, "key expansion", 0, NULL, 34, (uint8_t *)KEY_EXP_VALUE);
-	TRY(6, "hello", 0, NULL, 31, (uint8_t *)HELLO_VALUE);
-	TRY(7, "context", 5, "abcd\xfa", 31, (uint8_t *)CONTEXT_VALUE);
-	TRY(12, "null-context", 0, "", 31, (uint8_t *)NULL_CONTEXT_VALUE);
+	TRY(13, "key expansion", 0, NULL, 34, (uint8_t*)KEY_EXP_VALUE);
+	TRY(6, "hello", 0, NULL, 31, (uint8_t*)HELLO_VALUE);
+	TRY(7, "context", 5, "abcd\xfa", 31, (uint8_t*)CONTEXT_VALUE);
+	TRY(12, "null-context", 0, "", 31, (uint8_t*)NULL_CONTEXT_VALUE);
 
 	/* Try whether calling gnutls_prf() with non-null context or server-first
 	 * param, will fail */
-	ret = gnutls_prf(session, 3, (void *)"xxx", 0, 3, (void *)"yyy", 16,
-			 (void *)key_material);
+	ret = gnutls_prf(session, 3, (void*)"xxx", 0, 3, (void*)"yyy", 16, (void*)key_material);
 	if (ret != GNUTLS_E_INVALID_REQUEST)
 		fail("gnutls_prf: succeeded under TLS1.3!\n");
 
-	ret = gnutls_prf(session, 3, (void *)"xxx", 1, 0, NULL, 16,
-			 (void *)key_material);
+	ret = gnutls_prf(session, 3, (void*)"xxx", 1, 0, NULL, 16, (void*)key_material);
 	if (ret != GNUTLS_E_INVALID_REQUEST)
 		fail("gnutls_prf: succeeded under TLS1.3!\n");
 }
@@ -203,12 +184,10 @@ static void client(int fd)
 	 */
 	gnutls_init(&session, GNUTLS_CLIENT);
 
-	/* Use default priorities, sets %NO_SHUFFLE_EXTENSIONS */
-	ret = gnutls_priority_set_direct(
-		session,
-		"NONE:+VERS-TLS1.3:+AES-256-GCM:+AEAD:+SIGN-RSA-PSS-RSAE-SHA384:+GROUP-SECP256R1:%NO_SHUFFLE_EXTENSIONS",
-		&err);
-
+	/* Use default priorities */
+	ret = gnutls_priority_set_direct(session,
+				   "NONE:+VERS-TLS1.3:+AES-256-GCM:+AEAD:+SIGN-RSA-PSS-RSAE-SHA384:+GROUP-SECP256R1",
+				   &err);
 	if (ret < 0) {
 		fail("client: priority set failed (%s): %s\n",
 		     gnutls_strerror(ret), err);
@@ -216,7 +195,7 @@ static void client(int fd)
 	}
 
 	ret = gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE,
-				     clientx509cred);
+				clientx509cred);
 	if (ret < 0)
 		exit(1);
 
@@ -227,7 +206,8 @@ static void client(int fd)
 	 */
 	do {
 		ret = gnutls_handshake(session);
-	} while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
+	}
+	while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
 
 	if (ret < 0) {
 		fail("client: Handshake failed: %s\n", strerror(ret));
@@ -239,27 +219,18 @@ static void client(int fd)
 
 	if (debug)
 		success("client: TLS version is: %s\n",
-			gnutls_protocol_get_name(
-				gnutls_protocol_get_version(session)));
+			gnutls_protocol_get_name
+			(gnutls_protocol_get_version(session)));
 
 	ret = gnutls_cipher_get(session);
 	if (ret != GNUTLS_CIPHER_AES_256_GCM) {
-		fprintf(stderr, "negotiated unexpected cipher: %s\n",
-			gnutls_cipher_get_name(ret));
+		fprintf(stderr, "negotiated unexpected cipher: %s\n", gnutls_cipher_get_name(ret));
 		exit(1);
 	}
 
 	ret = gnutls_mac_get(session);
 	if (ret != GNUTLS_MAC_AEAD) {
-		fprintf(stderr, "negotiated unexpected mac: %s\n",
-			gnutls_mac_get_name(ret));
-		exit(1);
-	}
-
-	ret = gnutls_prf_hash_get(session);
-	if (ret != GNUTLS_DIG_SHA384) {
-		fprintf(stderr, "negotiated unexpected hash: %s\n",
-			gnutls_digest_get_name(ret));
+		fprintf(stderr, "negotiated unexpected mac: %s\n", gnutls_mac_get_name(ret));
 		exit(1);
 	}
 
@@ -280,10 +251,8 @@ static void terminate(void)
 {
 	int status = 0;
 
-	if (child) {
-		kill(child, SIGTERM);
-		wait(&status);
-	}
+	kill(child, SIGTERM);
+	wait(&status);
 	exit(1);
 }
 
@@ -309,26 +278,27 @@ static void server(int fd)
 	/* avoid calling all the priority functions, since the defaults
 	 * are adequate.
 	 */
-	ret = gnutls_priority_set_direct(
-		session,
-		"NORMAL:-VERS-ALL:+VERS-TLS1.3:-KX-ALL:-SIGN-ALL:+SIGN-RSA-PSS-RSAE-SHA384:-GROUP-ALL:+GROUP-SECP256R1",
-		NULL);
+	ret = gnutls_priority_set_direct(session,
+				   "NORMAL:-VERS-ALL:+VERS-TLS1.3:-KX-ALL:-SIGN-ALL:+SIGN-RSA-PSS-RSAE-SHA384:-GROUP-ALL:+GROUP-SECP256R1", NULL);
 	if (ret < 0) {
 		fail("server: priority set failed (%s)\n\n",
 		     gnutls_strerror(ret));
 		terminate();
 	}
 
-	gnutls_certificate_set_x509_key_mem(serverx509cred, &server_cert,
-					    &server_key, GNUTLS_X509_FMT_PEM);
-	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, serverx509cred);
+	gnutls_certificate_set_x509_key_mem(serverx509cred,
+					    &server_cert, &server_key,
+					    GNUTLS_X509_FMT_PEM);
+	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE,
+				serverx509cred);
 
 	gnutls_handshake_set_random(session, &hsrnd);
 	gnutls_transport_set_int(session, fd);
 
 	do {
 		ret = gnutls_handshake(session);
-	} while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
+	}
+	while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
 	if (ret < 0) {
 		close(fd);
 		gnutls_deinit(session);
@@ -341,10 +311,8 @@ static void server(int fd)
 
 	if (debug)
 		success("server: TLS version is: %s\n",
-			gnutls_protocol_get_name(
-				gnutls_protocol_get_version(session)));
-
-	check_prfs(session);
+			gnutls_protocol_get_name
+			(gnutls_protocol_get_version(session)));
 
 	/* do not wait for the peer to close the connection.
 	 */
@@ -395,4 +363,4 @@ void doit(void)
 	}
 }
 
-#endif /* _WIN32 */
+#endif				/* _WIN32 */
